@@ -2,7 +2,7 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017-2020 Osimis S.A., Belgium
+ * Copyright (C) 2017-2021 Osimis S.A., Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
@@ -29,44 +29,44 @@
   Program: DCMTK 3.6.0
   Module:  http://dicom.offis.de/dcmtk.php.en
 
-Copyright (C) 1994-2011, OFFIS e.V.
-All rights reserved.
+  Copyright (C) 1994-2011, OFFIS e.V.
+  All rights reserved.
 
-This software and supporting documentation were developed by
+  This software and supporting documentation were developed by
 
   OFFIS e.V.
   R&D Division Health
   Escherweg 2
   26121 Oldenburg, Germany
 
-Redistribution and use in source and binary forms, with or without
-modification, are permitted provided that the following conditions
-are met:
+  Redistribution and use in source and binary forms, with or without
+  modification, are permitted provided that the following conditions
+  are met:
 
-- Redistributions of source code must retain the above copyright
+  - Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.
 
-- Redistributions in binary form must reproduce the above copyright
+  - Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.
 
-- Neither the name of OFFIS nor the names of its contributors may be
+  - Neither the name of OFFIS nor the names of its contributors may be
   used to endorse or promote products derived from this software
   without specific prior written permission.
 
-THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
-"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
-LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
-A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
-HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
-LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
-DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
-THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
-(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
+  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
+  HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
+  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
+  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
+  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
+  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
+  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
+  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
-=========================================================================*/
+  =========================================================================*/
 
 
 #include "../../PrecompiledHeaders.h"
@@ -76,14 +76,15 @@ OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 #  error The macro DCMTK_VERSION_NUMBER must be defined
 #endif
 
-#include "FindScp.h"
-#include "StoreScp.h"
-#include "MoveScp.h"
-#include "GetScp.h"
 #include "../../Compatibility.h"
-#include "../../Toolbox.h"
+#include "../../DicomParsing/FromDcmtkBridge.h"
 #include "../../Logging.h"
 #include "../../OrthancException.h"
+#include "../../Toolbox.h"
+#include "FindScp.h"
+#include "GetScp.h"
+#include "MoveScp.h"
+#include "StoreScp.h"
 
 #include <dcmtk/dcmdata/dcdeftag.h>     /* for storage commitment */
 #include <dcmtk/dcmdata/dcsequen.h>     /* for class DcmSequenceOfItems */
@@ -92,7 +93,7 @@ OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
 #include <boost/lexical_cast.hpp>
 
-  static OFBool    opt_rejectWithoutImplementationUID = OFFalse;
+static OFBool    opt_rejectWithoutImplementationUID = OFFalse;
 
 
 
@@ -137,20 +138,17 @@ static OFCondition acceptUnknownContextsWithTransferSyntax(
   const char* transferSyntax,
   T_ASC_SC_ROLE acceptedRole)
 {
-  OFCondition cond = EC_Normal;
   int n, i, k;
   DUL_PRESENTATIONCONTEXT *dpc;
   T_ASC_PresentationContext pc;
-  OFBool accepted = OFFalse;
-  OFBool abstractOK = OFFalse;
 
   n = ASC_countPresentationContexts(params);
   for (i = 0; i < n; i++)
   {
-    cond = ASC_getPresentationContext(params, i, &pc);
+    OFCondition cond = ASC_getPresentationContext(params, i, &pc);
     if (cond.bad()) return cond;
-    abstractOK = OFFalse;
-    accepted = OFFalse;
+    OFBool abstractOK = OFFalse;
+    OFBool accepted = OFFalse;
 
     if (dcmFindNameOfUID(pc.abstractSyntax) == NULL)
     {
@@ -178,7 +176,7 @@ static OFCondition acceptUnknownContextsWithTransferSyntax(
       /* do not refuse if already accepted */
       dpc = findPresentationContextID(params->DULparams.acceptedPresentationContext,
                                       pc.presentationContextID);
-      if ((dpc == NULL) || ((dpc != NULL) && (dpc->result != ASC_P_ACCEPTANCE)))
+      if (dpc == NULL || dpc->result != ASC_P_ACCEPTANCE)
       {
 
         if (abstractOK) {
@@ -240,14 +238,14 @@ namespace Orthanc
       OFCondition cond = ASC_dropSCPAssociation(assoc);
       if (cond.bad())
       {
-        LOG(ERROR) << cond.text();
+        CLOG(ERROR, DICOM) << cond.text();
         return cond;
       }
 
       cond = ASC_destroyAssociation(&assoc);
       if (cond.bad())
       {
-        LOG(ERROR) << cond.text();
+        CLOG(ERROR, DICOM) << cond.text();
         return cond;
       }
 
@@ -256,7 +254,10 @@ namespace Orthanc
 
 
 
-    CommandDispatcher* AcceptAssociation(const DicomServer& server, T_ASC_Network *net)
+    CommandDispatcher* AcceptAssociation(const DicomServer& server,
+                                         T_ASC_Network *net,
+                                         unsigned int maximumPduLength,
+                                         bool useDicomTls)
     {
       DcmAssociationConfiguration asccfg;
       char buf[BUFSIZ];
@@ -265,10 +266,8 @@ namespace Orthanc
       OFString sprofile;
       OFString temp_str;
 
-      cond = ASC_receiveAssociation(net, &assoc, 
-                                    /*opt_maxPDU*/ ASC_DEFAULTMAXPDU, 
-                                    NULL, NULL,
-                                    /*opt_secureConnection*/ OFFalse,
+      cond = ASC_receiveAssociation(net, &assoc, maximumPduLength, NULL, NULL,
+                                    useDicomTls /*opt_secureConnection*/,
                                     DUL_NOBLOCK, 1);
 
       if (cond == DUL_NOASSOCIATIONREQUEST)
@@ -281,12 +280,18 @@ namespace Orthanc
       // if some kind of error occured, take care of it
       if (cond.bad())
       {
-        LOG(ERROR) << "Receiving Association failed: " << cond.text();
+        CLOG(ERROR, DICOM) << "Receiving Association failed: " << cond.text();
         // no matter what kind of error occurred, we need to do a cleanup
         AssociationCleanup(assoc);
         return NULL;
       }
 
+      {
+        OFString str;
+        CLOG(TRACE, DICOM) << "Received Association Parameters:" << std::endl
+                           << ASC_dumpParameters(str, assoc->params, ASC_ASSOC_RQ);
+      }
+
       // Retrieve the AET and the IP address of the remote modality
       std::string remoteAet;
       std::string remoteIp;
@@ -324,8 +329,8 @@ namespace Orthanc
         calledAet = (/*OFSTRING_GUARD*/(calledAet_C));
       }
 
-      LOG(INFO) << "Association Received from AET " << remoteAet 
-                << " on IP " << remoteIp;
+      CLOG(INFO, DICOM) << "Association Received from AET " << remoteAet 
+                        << " on IP " << remoteIp;
 
 
       {
@@ -375,7 +380,7 @@ namespace Orthanc
           &genericTransferSyntaxes[0], genericTransferSyntaxes.size());
         if (cond.bad())
         {
-          LOG(INFO) << cond.text();
+          CLOG(INFO, DICOM) << cond.text();
           AssociationCleanup(assoc);
           return NULL;
         }
@@ -395,7 +400,7 @@ namespace Orthanc
             &genericTransferSyntaxes[0], genericTransferSyntaxes.size(), ASC_SC_ROLE_SCUSCP);
           if (cond.bad())
           {
-            LOG(INFO) << cond.text();
+            CLOG(INFO, DICOM) << cond.text();
             AssociationCleanup(assoc);
             return NULL;
           }
@@ -405,185 +410,157 @@ namespace Orthanc
 
       {
         /* accept the abstract syntaxes for C-STORE, if presented */
+        
+        std::set<DicomTransferSyntax> storageTransferSyntaxes;
 
-        std::vector<const char*> storageTransferSyntaxes;
-
-        // This is the list of the transfer syntaxes that were supported up to Orthanc 0.7.1
-        storageTransferSyntaxes.push_back(UID_LittleEndianExplicitTransferSyntax);
-        storageTransferSyntaxes.push_back(UID_BigEndianExplicitTransferSyntax);
-        storageTransferSyntaxes.push_back(UID_LittleEndianImplicitTransferSyntax);
-
-        // New transfer syntaxes supported since Orthanc 0.7.2
-        if (!server.HasApplicationEntityFilter() ||
-            server.GetApplicationEntityFilter().IsAllowedTransferSyntax(remoteIp, remoteAet, calledAet, TransferSyntax_Deflated))
+        if (server.HasApplicationEntityFilter())
         {
-          storageTransferSyntaxes.push_back(UID_DeflatedExplicitVRLittleEndianTransferSyntax); 
+          server.GetApplicationEntityFilter().GetAcceptedTransferSyntaxes(
+            storageTransferSyntaxes, remoteIp, remoteAet, calledAet);
+        }
+        else
+        {
+          /**
+           * In the absence of filter, accept all the known transfer
+           * syntaxes. Note that this is different from Orthanc
+           * framework <= 1.8.2, where only the uncompressed transfer
+           * syntaxes are accepted by default.
+           **/
+          GetAllDicomTransferSyntaxes(storageTransferSyntaxes);
         }
 
-        if (!server.HasApplicationEntityFilter() ||
-            server.GetApplicationEntityFilter().IsAllowedTransferSyntax(remoteIp, remoteAet, calledAet, TransferSyntax_Jpeg))
+        if (storageTransferSyntaxes.empty())
         {
-          storageTransferSyntaxes.push_back(UID_JPEGProcess1TransferSyntax);
-          storageTransferSyntaxes.push_back(UID_JPEGProcess2_4TransferSyntax);
-          storageTransferSyntaxes.push_back(UID_JPEGProcess3_5TransferSyntax);
-          storageTransferSyntaxes.push_back(UID_JPEGProcess6_8TransferSyntax);
-          storageTransferSyntaxes.push_back(UID_JPEGProcess7_9TransferSyntax);
-          storageTransferSyntaxes.push_back(UID_JPEGProcess10_12TransferSyntax);
-          storageTransferSyntaxes.push_back(UID_JPEGProcess11_13TransferSyntax);
-          storageTransferSyntaxes.push_back(UID_JPEGProcess14TransferSyntax);
-          storageTransferSyntaxes.push_back(UID_JPEGProcess15TransferSyntax);
-          storageTransferSyntaxes.push_back(UID_JPEGProcess16_18TransferSyntax);
-          storageTransferSyntaxes.push_back(UID_JPEGProcess17_19TransferSyntax);
-          storageTransferSyntaxes.push_back(UID_JPEGProcess20_22TransferSyntax);
-          storageTransferSyntaxes.push_back(UID_JPEGProcess21_23TransferSyntax);
-          storageTransferSyntaxes.push_back(UID_JPEGProcess24_26TransferSyntax);
-          storageTransferSyntaxes.push_back(UID_JPEGProcess25_27TransferSyntax);
-          storageTransferSyntaxes.push_back(UID_JPEGProcess28TransferSyntax);
-          storageTransferSyntaxes.push_back(UID_JPEGProcess29TransferSyntax);
-          storageTransferSyntaxes.push_back(UID_JPEGProcess14SV1TransferSyntax);
+          LOG(WARNING) << "The DICOM server accepts no transfer syntax, thus C-STORE SCP is disabled";
         }
+        else
+        {
+          /**
+           * If accepted, put "Little Endian Explicit" at the first
+           * place in the accepted transfer syntaxes. This first place
+           * has an impact on the result of "getscu" (cf. integration
+           * test "test_getscu"). We choose "Little Endian Explicit",
+           * as this preserves the VR of the private tags, even if the
+           * remote modality doesn't have the dictionary of private tags.
+           *
+           * TODO - Should "PREFERRED_TRANSFER_SYNTAX" be an option of
+           * class "DicomServer"?
+           **/
+          const DicomTransferSyntax PREFERRED_TRANSFER_SYNTAX = DicomTransferSyntax_LittleEndianExplicit;
 
-        if (!server.HasApplicationEntityFilter() ||
-            server.GetApplicationEntityFilter().IsAllowedTransferSyntax(remoteIp, remoteAet, calledAet, TransferSyntax_Jpeg2000))
-        {
-          storageTransferSyntaxes.push_back(UID_JPEG2000LosslessOnlyTransferSyntax);
-          storageTransferSyntaxes.push_back(UID_JPEG2000TransferSyntax);
-          storageTransferSyntaxes.push_back(UID_JPEG2000LosslessOnlyTransferSyntax);
-          storageTransferSyntaxes.push_back(UID_JPEG2000TransferSyntax);
-          storageTransferSyntaxes.push_back(UID_JPEG2000Part2MulticomponentImageCompressionLosslessOnlyTransferSyntax);
-          storageTransferSyntaxes.push_back(UID_JPEG2000Part2MulticomponentImageCompressionTransferSyntax);
-        }
+          E_TransferSyntax dummy;
+          assert(FromDcmtkBridge::LookupDcmtkTransferSyntax(dummy, PREFERRED_TRANSFER_SYNTAX));
 
-        if (!server.HasApplicationEntityFilter() ||
-            server.GetApplicationEntityFilter().IsAllowedTransferSyntax(remoteIp, remoteAet, calledAet, TransferSyntax_JpegLossless))
-        {
-          storageTransferSyntaxes.push_back(UID_JPEGLSLosslessTransferSyntax);
-          storageTransferSyntaxes.push_back(UID_JPEGLSLossyTransferSyntax);
-        }
+          std::vector<const char*> storageTransferSyntaxesC;
+          storageTransferSyntaxesC.reserve(storageTransferSyntaxes.size());
 
-        if (!server.HasApplicationEntityFilter() ||
-            server.GetApplicationEntityFilter().IsAllowedTransferSyntax(remoteIp, remoteAet, calledAet, TransferSyntax_Jpip))
-        {
-          storageTransferSyntaxes.push_back(UID_JPIPReferencedTransferSyntax);
-          storageTransferSyntaxes.push_back(UID_JPIPReferencedDeflateTransferSyntax);
-        }
+          if (storageTransferSyntaxes.find(PREFERRED_TRANSFER_SYNTAX) != storageTransferSyntaxes.end())
+          {
+            storageTransferSyntaxesC.push_back(GetTransferSyntaxUid(PREFERRED_TRANSFER_SYNTAX));
+          }
+          
+          for (std::set<DicomTransferSyntax>::const_iterator
+                 syntax = storageTransferSyntaxes.begin(); syntax != storageTransferSyntaxes.end(); ++syntax)
+          {
+            if (*syntax != PREFERRED_TRANSFER_SYNTAX &&  // Don't add the preferred transfer syntax twice
+                // Make sure that the current version of DCMTK knows this transfer syntax
+                FromDcmtkBridge::LookupDcmtkTransferSyntax(dummy, *syntax))
+            {
+              storageTransferSyntaxesC.push_back(GetTransferSyntaxUid(*syntax));
+            }
+          }
 
-        if (!server.HasApplicationEntityFilter() ||
-            server.GetApplicationEntityFilter().IsAllowedTransferSyntax(remoteIp, remoteAet, calledAet, TransferSyntax_Mpeg2))
-        {
-          storageTransferSyntaxes.push_back(UID_MPEG2MainProfileAtMainLevelTransferSyntax);
-          storageTransferSyntaxes.push_back(UID_MPEG2MainProfileAtHighLevelTransferSyntax);
-        }
-
-#if DCMTK_VERSION_NUMBER >= 361
-        // New in Orthanc 1.6.0
-        if (!server.HasApplicationEntityFilter() ||
-            server.GetApplicationEntityFilter().IsAllowedTransferSyntax(remoteIp, remoteAet, calledAet, TransferSyntax_Mpeg4))
-        {
-          storageTransferSyntaxes.push_back(UID_MPEG4BDcompatibleHighProfileLevel4_1TransferSyntax);
-          storageTransferSyntaxes.push_back(UID_MPEG4HighProfileLevel4_1TransferSyntax);
-          storageTransferSyntaxes.push_back(UID_MPEG4HighProfileLevel4_2_For2DVideoTransferSyntax);
-          storageTransferSyntaxes.push_back(UID_MPEG4HighProfileLevel4_2_For3DVideoTransferSyntax);
-          storageTransferSyntaxes.push_back(UID_MPEG4StereoHighProfileLevel4_2TransferSyntax);
-        }
-#endif
-
-        if (!server.HasApplicationEntityFilter() ||
-            server.GetApplicationEntityFilter().IsAllowedTransferSyntax(remoteIp, remoteAet, calledAet, TransferSyntax_Rle))
-        {
-          storageTransferSyntaxes.push_back(UID_RLELosslessTransferSyntax);
-        }
-
-        /* the array of Storage SOP Class UIDs that is defined within "dcmdata/libsrc/dcuid.cc" */
-        size_t count = 0;
-        while (dcmAllStorageSOPClassUIDs[count] != NULL)
-        {
-          count++;
-        }
+          /* the array of Storage SOP Class UIDs that is defined within "dcmdata/libsrc/dcuid.cc" */
+          size_t count = 0;
+          while (dcmAllStorageSOPClassUIDs[count] != NULL)
+          {
+            count++;
+          }
         
 #if DCMTK_VERSION_NUMBER >= 362
-        // The global variable "numberOfDcmAllStorageSOPClassUIDs" is
-        // only published if DCMTK >= 3.6.2:
-        // https://bitbucket.org/sjodogne/orthanc/issues/137
-        assert(static_cast<int>(count) == numberOfDcmAllStorageSOPClassUIDs);
+          // The global variable "numberOfDcmAllStorageSOPClassUIDs" is
+          // only published if DCMTK >= 3.6.2:
+          // https://bugs.orthanc-server.com/show_bug.cgi?id=137
+          assert(static_cast<int>(count) == numberOfDcmAllStorageSOPClassUIDs);
 #endif
       
-        if (!server.HasGetRequestHandlerFactory())    // dcmqrsrv.cc line 828
-        {
-          // This branch exactly corresponds to Orthanc <= 1.6.1 (in
-          // which C-GET SCP was not supported)
-          cond = ASC_acceptContextsWithPreferredTransferSyntaxes(
-            assoc->params, dcmAllStorageSOPClassUIDs, count,
-            &storageTransferSyntaxes[0], storageTransferSyntaxes.size());
-          if (cond.bad())
+          if (!server.HasGetRequestHandlerFactory())    // dcmqrsrv.cc line 828
           {
-            LOG(INFO) << cond.text();
-            AssociationCleanup(assoc);
-            return NULL;
-          }
-        }
-        else                                         // see dcmqrsrv.cc lines 839 - 876
-        {
-          /* accept storage syntaxes with proposed role */
-          int npc = ASC_countPresentationContexts(assoc->params);
-          for (int i = 0; i < npc; i++)
-          {
-            T_ASC_PresentationContext pc;
-            ASC_getPresentationContext(assoc->params, i, &pc);
-            if (dcmIsaStorageSOPClassUID(pc.abstractSyntax))
+            // This branch exactly corresponds to Orthanc <= 1.6.1 (in
+            // which C-GET SCP was not supported)
+            cond = ASC_acceptContextsWithPreferredTransferSyntaxes(
+              assoc->params, dcmAllStorageSOPClassUIDs, count,
+              &storageTransferSyntaxesC[0], storageTransferSyntaxesC.size());
+            if (cond.bad())
             {
-              /**
-               * We are prepared to accept whatever role the caller
-               * proposes.  Normally we can be the SCP of the Storage
-               * Service Class.  When processing the C-GET operation
-               * we can be the SCU of the Storage Service Class.
-               **/
-              const T_ASC_SC_ROLE role = pc.proposedRole;
-            
-              /**
-               * Accept in the order "least wanted" to "most wanted"
-               * transfer syntax.  Accepting a transfer syntax will
-               * override previously accepted transfer syntaxes.
-               **/
-              for (int k = static_cast<int>(storageTransferSyntaxes.size()) - 1; k >= 0; k--)
+              CLOG(INFO, DICOM) << cond.text();
+              AssociationCleanup(assoc);
+              return NULL;
+            }
+          }
+          else                                         // see dcmqrsrv.cc lines 839 - 876
+          {
+            /* accept storage syntaxes with proposed role */
+            int npc = ASC_countPresentationContexts(assoc->params);
+            for (int i = 0; i < npc; i++)
+            {
+              T_ASC_PresentationContext pc;
+              ASC_getPresentationContext(assoc->params, i, &pc);
+              if (dcmIsaStorageSOPClassUID(pc.abstractSyntax))
               {
-                for (int j = 0; j < static_cast<int>(pc.transferSyntaxCount); j++)
+                /**
+                 * We are prepared to accept whatever role the caller
+                 * proposes.  Normally we can be the SCP of the Storage
+                 * Service Class.  When processing the C-GET operation
+                 * we can be the SCU of the Storage Service Class.
+                 **/
+                const T_ASC_SC_ROLE role = pc.proposedRole;
+            
+                /**
+                 * Accept in the order "least wanted" to "most wanted"
+                 * transfer syntax.  Accepting a transfer syntax will
+                 * override previously accepted transfer syntaxes.
+                 **/
+                for (int k = static_cast<int>(storageTransferSyntaxesC.size()) - 1; k >= 0; k--)
                 {
-                  /**
-                   * If the transfer syntax was proposed then we can accept it
-                   * appears in our supported list of transfer syntaxes
-                   **/
-                  if (strcmp(pc.proposedTransferSyntaxes[j], storageTransferSyntaxes[k]) == 0)
+                  for (int j = 0; j < static_cast<int>(pc.transferSyntaxCount); j++)
                   {
-                    cond = ASC_acceptPresentationContext(
-                      assoc->params, pc.presentationContextID, storageTransferSyntaxes[k], role);
-                    if (cond.bad())
+                    /**
+                     * If the transfer syntax was proposed then we can accept it
+                     * appears in our supported list of transfer syntaxes
+                     **/
+                    if (strcmp(pc.proposedTransferSyntaxes[j], storageTransferSyntaxesC[k]) == 0)
                     {
-                      LOG(INFO) << cond.text();
-                      AssociationCleanup(assoc);
-                      return NULL;
+                      cond = ASC_acceptPresentationContext(
+                        assoc->params, pc.presentationContextID, storageTransferSyntaxesC[k], role);
+                      if (cond.bad())
+                      {
+                        CLOG(INFO, DICOM) << cond.text();
+                        AssociationCleanup(assoc);
+                        return NULL;
+                      }
                     }
                   }
                 }
               }
-            }
-          } /* for */
-        }
+            } /* for */
+          }
 
-        if (!server.HasApplicationEntityFilter() ||
-            server.GetApplicationEntityFilter().IsUnknownSopClassAccepted(remoteIp, remoteAet, calledAet))
-        {
-          /*
-           * Promiscous mode is enabled: Accept everything not known not
-           * to be a storage SOP class.
-           **/
-          cond = acceptUnknownContextsWithPreferredTransferSyntaxes(
-            assoc->params, &storageTransferSyntaxes[0], storageTransferSyntaxes.size(), ASC_SC_ROLE_DEFAULT);
-          if (cond.bad())
+          if (!server.HasApplicationEntityFilter() ||
+              server.GetApplicationEntityFilter().IsUnknownSopClassAccepted(remoteIp, remoteAet, calledAet))
           {
-            LOG(INFO) << cond.text();
-            AssociationCleanup(assoc);
-            return NULL;
+            /*
+             * Promiscous mode is enabled: Accept everything not known not
+             * to be a storage SOP class.
+             **/
+            cond = acceptUnknownContextsWithPreferredTransferSyntaxes(
+              assoc->params, &storageTransferSyntaxesC[0], storageTransferSyntaxesC.size(), ASC_SC_ROLE_DEFAULT);
+            if (cond.bad())
+            {
+              CLOG(INFO, DICOM) << cond.text();
+              AssociationCleanup(assoc);
+              return NULL;
+            }
           }
         }
       }
@@ -608,11 +585,11 @@ namespace Orthanc
             ASC_REASON_SU_APPCONTEXTNAMENOTSUPPORTED
           };
 
-        LOG(INFO) << "Association Rejected: Bad Application Context Name: " << buf;
+        CLOG(INFO, DICOM) << "Association Rejected: Bad Application Context Name: " << buf;
         cond = ASC_rejectAssociation(assoc, &rej);
         if (cond.bad())
         {
-          LOG(INFO) << cond.text();
+          CLOG(INFO, DICOM) << cond.text();
         }
         AssociationCleanup(assoc);
         return NULL;
@@ -621,7 +598,7 @@ namespace Orthanc
       /* check the AETs */
       if (!server.IsMyAETitle(calledAet))
       {
-        LOG(WARNING) << "Rejected association, because of a bad called AET in the request (" << calledAet << ")";
+        CLOG(WARNING, DICOM) << "Rejected association, because of a bad called AET in the request (" << calledAet << ")";
         T_ASC_RejectParameters rej =
           {
             ASC_RESULT_REJECTEDPERMANENT,
@@ -636,7 +613,7 @@ namespace Orthanc
       if (server.HasApplicationEntityFilter() &&
           !server.GetApplicationEntityFilter().IsAllowedConnection(remoteIp, remoteAet, calledAet))
       {
-        LOG(WARNING) << "Rejected association for remote AET " << remoteAet << " on IP " << remoteIp;
+        CLOG(WARNING, DICOM) << "Rejected association for remote AET " << remoteAet << " on IP " << remoteIp;
         T_ASC_RejectParameters rej =
           {
             ASC_RESULT_REJECTEDPERMANENT,
@@ -659,11 +636,11 @@ namespace Orthanc
             ASC_REASON_SU_NOREASON
           };
 
-        LOG(INFO) << "Association Rejected: No Implementation Class UID provided";
+        CLOG(INFO, DICOM) << "Association Rejected: No Implementation Class UID provided";
         cond = ASC_rejectAssociation(assoc, &rej);
         if (cond.bad())
         {
-          LOG(INFO) << cond.text();
+          CLOG(INFO, DICOM) << cond.text();
         }
         AssociationCleanup(assoc);
         return NULL;
@@ -673,17 +650,29 @@ namespace Orthanc
         cond = ASC_acknowledgeAssociation(assoc);
         if (cond.bad())
         {
-          LOG(ERROR) << cond.text();
+          CLOG(ERROR, DICOM) << cond.text();
           AssociationCleanup(assoc);
           return NULL;
         }
-        LOG(INFO) << "Association Acknowledged (Max Send PDV: " << assoc->sendPDVLength << ")";
-        if (ASC_countAcceptedPresentationContexts(assoc->params) == 0)
-          LOG(INFO) << "    (but no valid presentation contexts)";
+
+        {
+          std::string suffix;
+          if (ASC_countAcceptedPresentationContexts(assoc->params) == 0)
+            suffix = " (but no valid presentation contexts)";
+          
+          CLOG(INFO, DICOM) << "Association Acknowledged (Max Send PDV: " << assoc->sendPDVLength
+                            << ") to AET " << remoteAet << " on IP " << remoteIp << suffix;
+        }
+
+        {
+          OFString str;
+          CLOG(TRACE, DICOM) << "Association Acknowledged Details:" << std::endl
+                             << ASC_dumpParameters(str, assoc->params, ASC_ASSOC_AC);
+        }
       }
 
       IApplicationEntityFilter* filter = server.HasApplicationEntityFilter() ? &server.GetApplicationEntityFilter() : NULL;
-      return new CommandDispatcher(server, assoc, remoteIp, remoteAet, calledAet, filter);
+      return new CommandDispatcher(server, assoc, remoteIp, remoteAet, calledAet, maximumPduLength, filter);
     }
 
 
@@ -692,6 +681,7 @@ namespace Orthanc
                                          const std::string& remoteIp,
                                          const std::string& remoteAet,
                                          const std::string& calledAet,
+                                         unsigned int maximumPduLength,
                                          IApplicationEntityFilter* filter) :
       server_(server),
       assoc_(assoc),
@@ -713,7 +703,7 @@ namespace Orthanc
       }
       catch (...)
       {
-        LOG(ERROR) << "Some association was not cleanly aborted";
+        CLOG(ERROR, DICOM) << "Some association was not cleanly aborted";
       }
     }
 
@@ -739,7 +729,10 @@ namespace Orthanc
       // detail information, dump this information
       if (statusDetail != NULL)
       {
-        //LOG4CPP_WARN(Internals::GetLogger(), "Status Detail:" << OFendl << DcmObject::PrintHelper(*statusDetail));
+        std::stringstream s;  // DcmObject::PrintHelper cannot be used with VS2008
+        statusDetail->print(s);
+        CLOG(TRACE, DICOM) << "Status Detail:" << std::endl << s.str();
+
         delete statusDetail;
       }
 
@@ -759,6 +752,12 @@ namespace Orthanc
       }
       else if (cond == EC_Normal)
       {
+        {
+          OFString str;
+          CLOG(TRACE, DICOM) << "Received Command:" << std::endl
+                             << DIMSE_dumpMessage(str, msg, DIMSE_INCOMING, NULL, presID);
+        }
+        
         // Reset the association timeout counter
         elapsedTimeSinceLastCommand_ = 0;
 
@@ -805,7 +804,7 @@ namespace Orthanc
           default:
             // we cannot handle this kind of message
             cond = DIMSE_BADCOMMANDTYPE;
-            LOG(ERROR) << "cannot handle command: 0x" << std::hex << msg.CommandField;
+            CLOG(ERROR, DICOM) << "cannot handle command: 0x" << std::hex << msg.CommandField;
             break;
         }
 
@@ -815,9 +814,9 @@ namespace Orthanc
             filter_ != NULL &&
             !filter_->IsAllowedRequest(remoteIp_, remoteAet_, calledAet_, request))
         {
-          LOG(WARNING) << "Rejected " << EnumerationToString(request)
-                       << " request from remote DICOM modality with AET \""
-                       << remoteAet_ << "\" and hostname \"" << remoteIp_ << "\"";
+          CLOG(WARNING, DICOM) << "Rejected " << EnumerationToString(request)
+                               << " request from remote DICOM modality with AET \""
+                               << remoteAet_ << "\" and hostname \"" << remoteIp_ << "\"";
           cond = DIMSE_ILLEGALASSOCIATION;
           supported = false;
           finished = true;
@@ -890,8 +889,7 @@ namespace Orthanc
                   worklistHandler.reset(server_.GetWorklistRequestHandlerFactory().ConstructWorklistRequestHandler());
                 }
 
-                cond = Internals::findScp(assoc_, &msg, presID, server_.GetRemoteModalities(),
-                                          findHandler.get(), worklistHandler.get(),
+                cond = Internals::findScp(assoc_, &msg, presID, findHandler.get(), worklistHandler.get(),
                                           remoteIp_, remoteAet_, calledAet_, associationTimeout_);
               }
               break;
@@ -916,24 +914,26 @@ namespace Orthanc
         // the peer or a network error
         finished = true;
 
-        LOG(INFO) << cond.text();
+        CLOG(INFO, DICOM) << "Finishing association with AET " << remoteAet_
+                          << " on IP " << remoteIp_ << ": " << cond.text();
       }
     
       if (finished)
       {
         if (cond == DUL_PEERREQUESTEDRELEASE)
         {
-          LOG(INFO) << "Association Release";
+          CLOG(INFO, DICOM) << "Association Release with AET " << remoteAet_ << " on IP " << remoteIp_;
           ASC_acknowledgeRelease(assoc_);
         }
         else if (cond == DUL_PEERABORTEDASSOCIATION)
         {
-          LOG(INFO) << "Association Aborted";
+          CLOG(INFO, DICOM) << "Association Aborted with AET " << remoteAet_ << " on IP " << remoteIp_;
         }
         else
         {
           OFString temp_str;
-          LOG(INFO) << "DIMSE failure (aborting association): " << cond.text();
+          CLOG(INFO, DICOM) << "DIMSE failure (aborting association with AET " << remoteAet_
+                            << " on IP " << remoteIp_ << "): " << cond.text();
           /* some kind of error so abort the association */
           ASC_abortAssociation(assoc_);
         }
@@ -946,14 +946,13 @@ namespace Orthanc
     OFCondition EchoScp(T_ASC_Association * assoc, T_DIMSE_Message * msg, T_ASC_PresentationContextID presID)
     {
       OFString temp_str;
-      LOG(INFO) << "Received Echo Request";
-      //LOG(DEBUG) << DIMSE_dumpMessage(temp_str, msg->msg.CEchoRQ, DIMSE_INCOMING, NULL, presID));
+      CLOG(INFO, DICOM) << "Received Echo Request";
 
       /* the echo succeeded !! */
       OFCondition cond = DIMSE_sendEchoResponse(assoc, presID, &msg->msg.CEchoRQ, STATUS_Success, NULL);
       if (cond.bad())
       {
-        LOG(ERROR) << "Echo SCP Failed: " << cond.text();
+        CLOG(ERROR, DICOM) << "Echo SCP Failed: " << cond.text();
       }
       return cond;
     }
@@ -1123,20 +1122,27 @@ namespace Orthanc
       
       std::unique_ptr<DcmDataset> dataset(
         ReadDataset(assoc_, "Cannot read the dataset in N-ACTION SCP", associationTimeout_));
+      assert(dataset.get() != NULL);
 
+      {
+        std::stringstream s;  // DcmObject::PrintHelper cannot be used with VS2008
+        dataset->print(s);
+        CLOG(TRACE, DICOM) << "Received Storage Commitment Request:" << std::endl << s.str();
+      }
+      
       std::string transactionUid = ReadString(*dataset, DCM_TransactionUID);
 
       std::vector<std::string> sopClassUid, sopInstanceUid;
       ReadSopSequence(sopClassUid, sopInstanceUid, NULL,
                       *dataset, DCM_ReferencedSOPSequence, true /* mandatory */);
 
-      LOG(INFO) << "Incoming storage commitment request, with transaction UID: " << transactionUid;
+      CLOG(INFO, DICOM) << "Incoming storage commitment request, with transaction UID: " << transactionUid;
 
       for (size_t i = 0; i < sopClassUid.size(); i++)
       {
-        LOG(INFO) << "  (" << (i + 1) << "/" << sopClassUid.size()
-                  << ") queried SOP Class/Instance UID: "
-                  << sopClassUid[i] << " / " << sopInstanceUid[i];
+        CLOG(INFO, DICOM) << "  (" << (i + 1) << "/" << sopClassUid.size()
+                          << ") queried SOP Class/Instance UID: "
+                          << sopClassUid[i] << " / " << sopInstanceUid[i];
       }
 
 
@@ -1161,7 +1167,7 @@ namespace Orthanc
       }
       catch (OrthancException& e)
       {
-        LOG(ERROR) << "Error while processing an incoming storage commitment request: " << e.What();
+        CLOG(ERROR, DICOM) << "Error while processing an incoming storage commitment request: " << e.What();
 
         // Code 0x0110 - "General failure in processing the operation was encountered"
         dimseStatus = STATUS_N_ProcessingFailure;
@@ -1186,6 +1192,12 @@ namespace Orthanc
         content.DataSetType = DIMSE_DATASET_NULL;  // Dataset is absent in storage commitment response
         content.opts = O_NACTION_AFFECTEDSOPCLASSUID | O_NACTION_AFFECTEDSOPINSTANCEUID;
 
+        {
+          OFString str;
+          CLOG(TRACE, DICOM) << "Sending Storage Commitment Request Response:" << std::endl
+                             << DIMSE_dumpMessage(str, response, DIMSE_OUTGOING);
+        }
+        
         return DIMSE_sendMessageUsingMemoryData(
           assoc_, presID, &response, NULL /* no dataset */, NULL /* dataObject */,
           NULL /* callback */, NULL /* callback context */, NULL /* commandSet */);
@@ -1246,7 +1258,14 @@ namespace Orthanc
       
       std::unique_ptr<DcmDataset> dataset(
         ReadDataset(assoc_, "Cannot read the dataset in N-EVENT-REPORT SCP", associationTimeout_));
+      assert(dataset.get() != NULL);
 
+      {
+        std::stringstream s;  // DcmObject::PrintHelper cannot be used with VS2008
+        dataset->print(s);
+        CLOG(TRACE, DICOM) << "Received Storage Commitment Report:" << std::endl << s.str();
+      }
+      
       std::string transactionUid = ReadString(*dataset, DCM_TransactionUID);
 
       std::vector<std::string> successSopClassUid, successSopInstanceUid;
@@ -1263,20 +1282,20 @@ namespace Orthanc
                         *dataset, DCM_FailedSOPSequence, true);
       }
 
-      LOG(INFO) << "Incoming storage commitment report, with transaction UID: " << transactionUid;
+      CLOG(INFO, DICOM) << "Incoming storage commitment report, with transaction UID: " << transactionUid;
 
       for (size_t i = 0; i < successSopClassUid.size(); i++)
       {
-        LOG(INFO) << "  (success " << (i + 1) << "/" << successSopClassUid.size()
-                  << ") SOP Class/Instance UID: "
-                  << successSopClassUid[i] << " / " << successSopInstanceUid[i];
+        CLOG(INFO, DICOM) << "  (success " << (i + 1) << "/" << successSopClassUid.size()
+                          << ") SOP Class/Instance UID: "
+                          << successSopClassUid[i] << " / " << successSopInstanceUid[i];
       }
 
       for (size_t i = 0; i < failedSopClassUid.size(); i++)
       {
-        LOG(INFO) << "  (failure " << (i + 1) << "/" << failedSopClassUid.size()
-                  << ") SOP Class/Instance UID: "
-                  << failedSopClassUid[i] << " / " << failedSopInstanceUid[i];
+        CLOG(INFO, DICOM) << "  (failure " << (i + 1) << "/" << failedSopClassUid.size()
+                          << ") SOP Class/Instance UID: "
+                          << failedSopClassUid[i] << " / " << failedSopInstanceUid[i];
       }
 
       /**
@@ -1301,7 +1320,7 @@ namespace Orthanc
       }
       catch (OrthancException& e)
       {
-        LOG(ERROR) << "Error while processing an incoming storage commitment report: " << e.What();
+        CLOG(ERROR, DICOM) << "Error while processing an incoming storage commitment report: " << e.What();
 
         // Code 0x0110 - "General failure in processing the operation was encountered"
         dimseStatus = STATUS_N_ProcessingFailure;
@@ -1326,6 +1345,12 @@ namespace Orthanc
         content.DataSetType = DIMSE_DATASET_NULL;  // Dataset is absent in storage commitment response
         content.opts = O_NEVENTREPORT_AFFECTEDSOPCLASSUID | O_NEVENTREPORT_AFFECTEDSOPINSTANCEUID;
 
+        {
+          OFString str;
+          CLOG(TRACE, DICOM) << "Sending Storage Commitment Report Response:" << std::endl
+                             << DIMSE_dumpMessage(str, response, DIMSE_OUTGOING);
+        }
+
         return DIMSE_sendMessageUsingMemoryData(
           assoc_, presID, &response, NULL /* no dataset */, NULL /* dataObject */,
           NULL /* callback */, NULL /* callback context */, NULL /* commandSet */);
