@@ -104,6 +104,7 @@
 #include <dcmtk/dcmdata/dcuid.h>
 #include <dcmtk/dcmdata/dcmetinf.h>
 #include <dcmtk/dcmdata/dcdeftag.h>
+#include <dcmtk/dcmdata/dcswap.h>
 
 #include <dcmtk/dcmdata/dcvrae.h>
 #include <dcmtk/dcmdata/dcvras.h>
@@ -419,28 +420,6 @@ namespace Orthanc
 #endif
 
   
-  static inline uint16_t GetCharValue(char c)
-  {
-    if (c >= '0' && c <= '9')
-      return c - '0';
-    else if (c >= 'a' && c <= 'f')
-      return c - 'a' + 10;
-    else if (c >= 'A' && c <= 'F')
-      return c - 'A' + 10;
-    else
-      return 0;
-  }
-
-  
-  static inline uint16_t GetTagValue(const char* c)
-  {
-    return ((GetCharValue(c[0]) << 12) + 
-            (GetCharValue(c[1]) << 8) + 
-            (GetCharValue(c[2]) << 4) + 
-            GetCharValue(c[3]));
-  }
-
-
 #if ORTHANC_ENABLE_CIVETWEB == 1 || ORTHANC_ENABLE_MONGOOSE == 1
   void ParsedDicomFile::SendPathValue(RestApiOutput& output,
                                       const UriComponents& uri) const
@@ -970,7 +949,10 @@ namespace Orthanc
 
   void ParsedDicomFile::SaveToMemoryBuffer(std::string& buffer)
   {
-    FromDcmtkBridge::SaveToMemoryBuffer(buffer, *GetDcmtkObject().getDataset());
+    if (!FromDcmtkBridge::SaveToMemoryBuffer(buffer, *GetDcmtkObject().getDataset()))
+    {
+      throw OrthancException(ErrorCode_InternalError, "Cannot write DICOM file to memory");
+    }
   }
 
 
@@ -1386,39 +1368,52 @@ namespace Orthanc
     const unsigned int height = accessor.GetHeight();
     const unsigned int width = accessor.GetWidth();
 
-    for (unsigned int y = 0; y < height; y++)
     {
-      switch (accessor.GetFormat())
+      Uint8* q = target;
+      for (unsigned int y = 0; y < height; y++)
       {
-        case PixelFormat_RGB24:
-        case PixelFormat_Grayscale8:
-        case PixelFormat_Grayscale16:
-        case PixelFormat_SignedGrayscale16:
+        switch (accessor.GetFormat())
         {
-          memcpy(target, reinterpret_cast<const Uint8*>(accessor.GetConstRow(y)), pitch);
-          target += pitch;
-          break;
-        }
-
-        case PixelFormat_RGBA32:
-        {
-          // The alpha channel is not supported by the DICOM standard
-          const Uint8* source = reinterpret_cast<const Uint8*>(accessor.GetConstRow(y));
-          for (unsigned int x = 0; x < width; x++, target += 3, source += 4)
+          case PixelFormat_RGB24:
+          case PixelFormat_Grayscale8:
+          case PixelFormat_Grayscale16:
+          case PixelFormat_SignedGrayscale16:
           {
-            target[0] = source[0];
-            target[1] = source[1];
-            target[2] = source[2];
+            memcpy(q, reinterpret_cast<const Uint8*>(accessor.GetConstRow(y)), pitch);
+            q += pitch;
+            break;
           }
 
-          break;
-        }
+          case PixelFormat_RGBA32:
+          {
+            // The alpha channel is not supported by the DICOM standard
+            const Uint8* source = reinterpret_cast<const Uint8*>(accessor.GetConstRow(y));
+            for (unsigned int x = 0; x < width; x++, q += 3, source += 4)
+            {
+              q[0] = source[0];
+              q[1] = source[1];
+              q[2] = source[2];
+            }
+
+            break;
+          }
           
-        default:
-          throw OrthancException(ErrorCode_NotImplemented);
+          default:
+            throw OrthancException(ErrorCode_NotImplemented);
+        }
       }
     }
 
+    static const Endianness ENDIANNESS = Toolbox::DetectEndianness();
+    if (ENDIANNESS == Endianness_Big &&
+        (accessor.GetFormat() == PixelFormat_Grayscale16 ||
+         accessor.GetFormat() == PixelFormat_SignedGrayscale16))
+    {
+      // New in Orthanc 1.9.1
+      assert(pitch % 2 == 0);
+      swapBytes(target, accessor.GetHeight() * pitch, sizeof(uint16_t));
+    }
+
     if (!GetDcmtkObject().getDataset()->insert(pixels.release(), false, false).good())
     {
       throw OrthancException(ErrorCode_InternalError);
@@ -1683,38 +1678,9 @@ namespace Orthanc
   }
 
 
-  bool ParsedDicomFile::LookupTransferSyntax(std::string& result) const
+  bool ParsedDicomFile::LookupTransferSyntax(DicomTransferSyntax& result) const
   {
-#if 0
-    // This was the implementation in Orthanc <= 1.6.1
-
-    // TODO - Shouldn't "dataset.getCurrentXfer()" be used instead of
-    // using the meta header?
-    const char* value = NULL;
-
-    if (GetDcmtkObjectConst().getMetaInfo() != NULL &&
-        GetDcmtkObjectConst().getMetaInfo()->findAndGetString(DCM_TransferSyntaxUID, value).good() &&
-        value != NULL)
-    {
-      result.assign(value);
-      return true;
-    }
-    else
-    {
-      return false;
-    }
-#else
-    DicomTransferSyntax s;
-    if (FromDcmtkBridge::LookupOrthancTransferSyntax(s, GetDcmtkObjectConst()))
-    {
-      result.assign(GetTransferSyntaxUid(s));
-      return true;
-    }
-    else
-    {
-      return false;
-    }
-#endif
+    return FromDcmtkBridge::LookupOrthancTransferSyntax(result, GetDcmtkObjectConst());
   }
 
 
@@ -1787,6 +1753,20 @@ namespace Orthanc
   {
     return const_cast<const ParsedDicomFile&>(*this).LookupTransferSyntax(result);
   }
+  
+  bool ParsedDicomFile::LookupTransferSyntax(std::string& result) const
+  {
+    DicomTransferSyntax s;
+    if (LookupTransferSyntax(s))
+    {
+      result = GetTransferSyntaxUid(s);
+      return true;
+    }
+    else
+    {
+      return false;
+    }
+  }
 
   bool ParsedDicomFile::GetTagValue(std::string& value,
                                     const DicomTag& tag)
