@@ -2,7 +2,7 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017-2020 Osimis S.A., Belgium
+ * Copyright (C) 2017-2021 Osimis S.A., Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
@@ -25,28 +25,44 @@
 #include "IHttpOutputStream.h"
 
 #include "../ChunkedBuffer.h"
+#include "../Compatibility.h"  // For ORTHANC_OVERRIDE
+
+#include <map>
+
 
 namespace Orthanc
 {
   class StringHttpOutput : public IHttpOutputStream
   {
   private:
-    bool          found_;
-    ChunkedBuffer buffer_;
+    HttpStatus    status_;
+    ChunkedBuffer body_;
+    ChunkedBuffer headers_;
+    bool          validBody_;
+    bool          validHeaders_;
 
   public:
-    StringHttpOutput() : found_(false)
+    StringHttpOutput();
+
+    virtual void OnHttpStatusReceived(HttpStatus status) ORTHANC_OVERRIDE
+    {
+      status_ = status;
+    }
+
+    virtual void Send(bool isHeader, const void* buffer, size_t length) ORTHANC_OVERRIDE;
+
+    virtual void DisableKeepAlive() ORTHANC_OVERRIDE
     {
     }
 
-    virtual void OnHttpStatusReceived(HttpStatus status);
-
-    virtual void Send(bool isHeader, const void* buffer, size_t length);
-
-    virtual void DisableKeepAlive()
+    HttpStatus GetStatus() const
     {
+      return status_;
     }
 
-    void GetOutput(std::string& output);
+    void GetBody(std::string& output);
+
+    void GetHeaders(std::map<std::string, std::string>& target,
+                    bool keyToLowerCase);
   };
 }
