@@ -2,7 +2,7 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017-2020 Osimis S.A., Belgium
+ * Copyright (C) 2017-2021 Osimis S.A., Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
@@ -25,30 +25,43 @@
 
 
 #if defined(_WIN32)
+#  include <winsock2.h>      // For GetMacAddresses(), must be included before "windows.h"
 #  include <windows.h>
-#  include <process.h>   // For "_spawnvp()" and "_getpid()"
-#  include <stdlib.h>    // For "environ"
+
+#  include <iphlpapi.h>      // For GetMacAddresses()
+#  include <process.h>       // For "_spawnvp()" and "_getpid()"
+#  include <stdlib.h>        // For "environ"
 #else
-#  include <unistd.h>    // For "execvp()"
-#  include <sys/wait.h>  // For "waitpid()"
+#  include <net/if.h>        // For GetMacAddresses()
+#  include <netinet/in.h>    // For GetMacAddresses()
+#  include <sys/ioctl.h>     // For GetMacAddresses()
+#  include <sys/wait.h>      // For "waitpid()"
+#  include <unistd.h>        // For "execvp()"
 #endif
 
 
 #if defined(__APPLE__) && defined(__MACH__)
-#  include <mach-o/dyld.h> /* _NSGetExecutablePath */
-#  include <limits.h>      /* PATH_MAX */
+#  include <limits.h>        // PATH_MAX
+#  include <mach-o/dyld.h>   // _NSGetExecutablePath
+#endif
+
+
+#if (defined(__APPLE__) && defined(__MACH__)) || defined(__FreeBSD_kernel__) || defined(__FreeBSD__)
+#  include <net/if_dl.h>     // For GetMacAddresses()
+#  include <net/if_types.h>  // For GetMacAddresses()
+#  include <sys/sysctl.h>    // For GetMacAddresses()
 #endif
 
 
 #if defined(__linux__) || defined(__FreeBSD_kernel__) || defined(__FreeBSD__)
-#  include <limits.h>      /* PATH_MAX */
+#  include <limits.h>        // PATH_MAX
 #  include <signal.h>
 #  include <unistd.h>
 #endif
 
 
 #if defined(__OpenBSD__)
-#  include <sys/sysctl.h>  // For "sysctl", "CTL_KERN" and "KERN_PROC_ARGS"
+#  include <sys/sysctl.h>    // For "sysctl", "CTL_KERN" and "KERN_PROC_ARGS"
 #endif
 
 
@@ -56,11 +69,17 @@
 #include "OrthancException.h"
 #include "Toolbox.h"
 
+#include <boost/iostreams/device/file_descriptor.hpp>
+#include <boost/iostreams/stream.hpp>
 #include <boost/filesystem.hpp>
 #include <boost/filesystem/fstream.hpp>
 #include <boost/date_time/posix_time/posix_time.hpp>
 #include <boost/thread.hpp>
 
+#include <cassert>
+#include <string.h>
+
+
 
 /*=========================================================================
   The section below comes from the Boost 1.68.0 project:
@@ -211,23 +230,47 @@ namespace Orthanc
                              log);
     }
 
-    boost::filesystem::ifstream f;
-    f.open(path, std::ifstream::in | std::ifstream::binary);
-    if (!f.good())
+    try
     {
-      throw OrthancException(ErrorCode_InexistentFile,
-                             "File not found: " + path,
-                             log);
-    }
+      boost::filesystem::ifstream f;
+      f.open(path, std::ifstream::in | std::ifstream::binary);
+      if (!f.good())
+      {
+        throw OrthancException(ErrorCode_InexistentFile,
+                               "File not found: " + path,
+                               log);
+      }
 
-    std::streamsize size = GetStreamSize(f);
-    content.resize(static_cast<size_t>(size));
-    if (size != 0)
+      std::streamsize size = GetStreamSize(f);
+      content.resize(static_cast<size_t>(size));
+
+      if (static_cast<std::streamsize>(content.size()) != size)
+      {
+        throw OrthancException(ErrorCode_InternalError,
+                               "Reading a file that is too large for a 32bit architecture");
+      }
+    
+      if (size != 0)
+      {
+        f.read(&content[0], size);
+      }
+
+      f.close();
+    }
+    catch (boost::filesystem::filesystem_error&)
     {
-      f.read(&content[0], size);
+      throw OrthancException(ErrorCode_InexistentFile);
     }
+    catch (...)  // To catch "std::system_error&" in C++11
+    {
+      throw OrthancException(ErrorCode_InexistentFile);
+    }
+  }
 
-    f.close();
+
+  void SystemToolbox::ReadFile(std::string &content, const std::string &path)
+  {
+    ReadFile(content, path, true /* log */);
   }
 
 
@@ -241,72 +284,139 @@ namespace Orthanc
                              "The path does not point to a regular file: " + path);
     }
 
-    boost::filesystem::ifstream f;
-    f.open(path, std::ifstream::in | std::ifstream::binary);
-    if (!f.good())
+    try
+    {
+      boost::filesystem::ifstream f;
+      f.open(path, std::ifstream::in | std::ifstream::binary);
+      if (!f.good())
+      {
+        throw OrthancException(ErrorCode_InexistentFile);
+      }
+
+      bool full = true;
+
+      {
+        std::streamsize size = GetStreamSize(f);
+        if (size <= 0)
+        {
+          headerSize = 0;
+          full = false;
+        }
+        else if (static_cast<size_t>(size) < headerSize)
+        {
+          headerSize = static_cast<size_t>(size);  // Truncate to the size of the file
+          full = false;
+        }
+      }
+
+      header.resize(headerSize);
+      if (headerSize != 0)
+      {
+        f.read(&header[0], headerSize);
+      }
+
+      f.close();
+
+      return full;
+    }
+    catch (boost::filesystem::filesystem_error&)
     {
       throw OrthancException(ErrorCode_InexistentFile);
     }
-
-    bool full = true;
-
+    catch (...)  // To catch "std::system_error&" in C++11
     {
-      std::streamsize size = GetStreamSize(f);
-      if (size <= 0)
-      {
-        headerSize = 0;
-        full = false;
-      }
-      else if (static_cast<size_t>(size) < headerSize)
-      {
-        headerSize = static_cast<size_t>(size);  // Truncate to the size of the file
-        full = false;
-      }
+      throw OrthancException(ErrorCode_InexistentFile);
     }
-
-    header.resize(headerSize);
-    if (headerSize != 0)
-    {
-      f.read(&header[0], headerSize);
-    }
-
-    f.close();
-
-    return full;
   }
 
 
   void SystemToolbox::WriteFile(const void* content,
                                 size_t size,
-                                const std::string& path)
+                                const std::string& path,
+                                bool callFsync)
   {
-    boost::filesystem::ofstream f;
-    f.open(path, std::ofstream::out | std::ofstream::binary);
-    if (!f.good())
+    try
+    {
+      //boost::filesystem::ofstream f;
+      boost::iostreams::stream<boost::iostreams::file_descriptor_sink> f;
+    
+      f.open(path, std::ofstream::out | std::ofstream::binary);
+      if (!f.good())
+      {
+        throw OrthancException(ErrorCode_CannotWriteFile);
+      }
+
+      if (size != 0)
+      {
+        f.write(reinterpret_cast<const char*>(content), size);
+
+        if (!f.good())
+        {
+          f.close();
+          throw OrthancException(ErrorCode_CannotWriteFile);
+        }
+      }
+
+      if (callFsync)
+      {
+        // https://stackoverflow.com/a/23826489/881731
+        f.flush();
+
+        bool success;
+
+        /**
+         * "f->handle()" corresponds to "FILE*" (aka "HANDLE") on
+         * Microsoft Windows, and to "int" (file descriptor) on other
+         * systems:
+         * https://github.com/boostorg/iostreams/blob/develop/include/boost/iostreams/detail/file_handle.hpp
+         **/
+      
+#if defined(_WIN32)
+        // https://docs.microsoft.com/fr-fr/windows/win32/api/fileapi/nf-fileapi-flushfilebuffers
+        success = (::FlushFileBuffers(f->handle()) != 0);
+#elif (_POSIX_C_SOURCE >= 199309L || _XOPEN_SOURCE >= 500)
+        success = (::fdatasync(f->handle()) == 0);
+#else
+        success = (::fsync(f->handle()) == 0);
+#endif
+
+        if (!success)
+        {
+          throw OrthancException(ErrorCode_CannotWriteFile, "Cannot force flush to disk");
+        }
+      }
+
+      f.close();
+    }
+    catch (boost::filesystem::filesystem_error&)
     {
       throw OrthancException(ErrorCode_CannotWriteFile);
     }
-
-    if (size != 0)
+    catch (...)  // To catch "std::system_error&" in C++11
     {
-      f.write(reinterpret_cast<const char*>(content), size);
-
-      if (!f.good())
-      {
-        f.close();
-        throw OrthancException(ErrorCode_FileStorageCannotWrite);
-      }
+      throw OrthancException(ErrorCode_CannotWriteFile);
     }
+  }
 
-    f.close();
+
+  void SystemToolbox::WriteFile(const void *content, size_t size, const std::string &path)
+  {
+    WriteFile(content, size, path, false /* don't automatically call fsync */);
   }
 
 
   void SystemToolbox::WriteFile(const std::string& content,
-                                const std::string& path)
+                                const std::string& path,
+                                bool callFsync)
   {
     WriteFile(content.size() > 0 ? content.c_str() : NULL,
-              content.size(), path);
+              content.size(), path, callFsync);
+  }
+
+
+  void SystemToolbox::WriteFile(const std::string &content, const std::string &path)
+  {
+    WriteFile(content, path, false /* don't automatically call fsync */);
   }
 
 
@@ -336,6 +446,10 @@ namespace Orthanc
     {
       throw OrthancException(ErrorCode_InexistentFile);
     }
+    catch (...)  // To catch "std::system_error&" in C++11
+    {
+      throw OrthancException(ErrorCode_InexistentFile);
+    }
   }
 
 
@@ -405,7 +519,7 @@ namespace Orthanc
   {
     // This is an adapted version of the patch proposed in issue #64
     // without an explicit call to "malloc()" to prevent memory leak
-    // https://bitbucket.org/sjodogne/orthanc/issues/64/add-openbsd-support
+    // https://bugs.orthanc-server.com/show_bug.cgi?id=64
     // https://stackoverflow.com/q/31494901/881731
 
     const int mib[4] = { CTL_KERN, KERN_PROC_ARGS, getpid(), KERN_PROC_ARGV };
@@ -515,18 +629,16 @@ namespace Orthanc
 
   bool SystemToolbox::IsRegularFile(const std::string& path)
   {
-    namespace fs = boost::filesystem;
-
     try
     {
-      if (fs::exists(path))
+      if (boost::filesystem::exists(path))
       {
-        fs::file_status status = fs::status(path);
+        boost::filesystem::file_status status = boost::filesystem::status(path);
         return (status.type() == boost::filesystem::regular_file ||
                 status.type() == boost::filesystem::reparse_file);   // Fix BitBucket issue #11
       }
     }
-    catch (fs::filesystem_error&)
+    catch (boost::filesystem::filesystem_error&)
     {
     }
 
@@ -602,7 +714,7 @@ namespace Orthanc
     // CPUs or cores or hyperthreading units)
     unsigned int threads = boost::thread::hardware_concurrency();
     
-    if (threads <= 0)
+    if (threads == 0)
     {
       return 1;
     }
@@ -665,6 +777,10 @@ namespace Orthanc
     }
 
     // Images types
+    else if (extension == ".dcm")
+    {
+      return MimeType_Dicom;
+    }
     else if (extension == ".jpg" ||
              extension == ".jpeg")
     {
@@ -750,4 +866,274 @@ namespace Orthanc
       return (base / relative).string();
     }
   }
+
+
+  void SystemToolbox::ReadFileRange(std::string& content,                              
+                                    const std::string& path,
+                                    uint64_t start,  // Inclusive
+                                    uint64_t end,    // Exclusive
+                                    bool throwIfOverflow)
+  {
+    if (start > end)
+    {
+      throw OrthancException(ErrorCode_ParameterOutOfRange);
+    }
+    
+    if (!IsRegularFile(path))
+    {
+      throw OrthancException(ErrorCode_RegularFileExpected,
+                             "The path does not point to a regular file: " + path);
+    }
+
+    boost::filesystem::ifstream f;
+    f.open(path, std::ifstream::in | std::ifstream::binary);
+    if (!f.good())
+    {
+      throw OrthancException(ErrorCode_InexistentFile,
+                             "File not found: " + path);
+    }
+
+    uint64_t fileSize = static_cast<uint64_t>(GetStreamSize(f));
+    if (end > fileSize)
+    {
+      if (throwIfOverflow)
+      {
+        throw OrthancException(ErrorCode_ParameterOutOfRange,
+                               "Reading beyond the end of a file");
+      }
+      else
+      {
+        end = fileSize;
+      }
+    }
+
+    if (start <= end)
+    {
+      content.resize(static_cast<size_t>(end - start));
+
+      if (static_cast<uint64_t>(content.size()) != (end - start))
+      {
+        throw OrthancException(ErrorCode_InternalError,
+                               "Reading a file that is too large for a 32bit architecture");
+      }
+
+      if (!content.empty())
+      {
+        f.seekg(start, std::ios::beg);
+        f.read(&content[0], static_cast<std::streamsize>(content.size()));
+      }
+    }
+    else
+    {
+      content.clear();
+    }
+
+    f.close();
+  }
+
+
+#if defined(_WIN32)
+  void SystemToolbox::GetMacAddresses(std::set<std::string>& target)
+  {
+    target.clear();
+    
+    // 15Ko is the recommanded size to start with
+    std::vector<char> buffer(15 * 1024);
+
+    for (unsigned int iteration = 0; iteration < 3; iteration++)
+    {
+      ULONG outBufLen = static_cast<ULONG>(buffer.size());
+      DWORD result = GetAdaptersAddresses
+        (AF_UNSPEC, 0, NULL, 
+         reinterpret_cast<IP_ADAPTER_ADDRESSES*>(&buffer[0]), &outBufLen);
+
+      if (result == NO_ERROR)
+      {
+        IP_ADAPTER_ADDRESSES* current =
+          reinterpret_cast<IP_ADAPTER_ADDRESSES*>(&buffer[0]); 
+
+        while (current != NULL)
+        {
+          if (current->PhysicalAddressLength == 6 &&
+              (current->PhysicalAddress[0] != 0 ||
+               current->PhysicalAddress[1] != 0 ||
+               current->PhysicalAddress[2] != 0 ||
+               current->PhysicalAddress[3] != 0 ||
+               current->PhysicalAddress[4] != 0 ||
+               current->PhysicalAddress[5] != 0))
+          {
+            char tmp[32];
+            sprintf(tmp, "%02x:%02x:%02x:%02x:%02x:%02x",
+                    (unsigned char) current->PhysicalAddress[0],
+                    (unsigned char) current->PhysicalAddress[1],
+                    (unsigned char) current->PhysicalAddress[2],
+                    (unsigned char) current->PhysicalAddress[3],
+                    (unsigned char) current->PhysicalAddress[4],
+                    (unsigned char) current->PhysicalAddress[5]);
+            target.insert(tmp);
+          }
+
+          current = current->Next;
+        }
+        
+        return;
+      }     
+      else if (result != ERROR_BUFFER_OVERFLOW || 
+               iteration >= 3 ||
+               outBufLen == 0)
+      {
+        return;
+      }
+      else
+      {
+        buffer.resize(outBufLen);
+        iteration++;
+      }
+    }
+  }
+
+#else
+  namespace
+  {
+    class SocketRaii : public boost::noncopyable
+    {
+    private:
+      int socket_;
+
+    public:
+      SocketRaii()
+      {
+        socket_ = socket(AF_INET, SOCK_DGRAM, IPPROTO_IP);
+      }
+
+      ~SocketRaii()
+      {
+        if (socket_ != -1)
+        {
+          close(socket_);
+        }
+      }
+
+      int GetDescriptor() const
+      {
+        return socket_;
+      }
+    };
+
+
+    class NetworkInterfaces : public boost::noncopyable
+    {
+    private:
+      struct if_nameindex* list_;
+      struct if_nameindex* current_;
+
+    public:
+      NetworkInterfaces()
+      {
+        list_ = if_nameindex();
+        current_ = list_;
+      }
+
+      ~NetworkInterfaces()
+      {
+        if (list_ != NULL)
+        {
+          if_freenameindex(list_);
+        }
+      }
+
+      bool IsDone() const
+      {
+        return (current_ == NULL ||
+                (current_->if_index == 0 &&
+                 current_->if_name == NULL));
+      }
+
+      const char* GetCurrentName() const
+      {
+        assert(!IsDone());
+        return current_->if_name;
+      }
+
+      unsigned int GetCurrentIndex() const
+      {
+        assert(!IsDone());
+        return current_->if_index;
+      }
+
+      void Next()
+      {
+        assert(!IsDone());
+        current_++;
+      }
+    };
+  }
+
+
+  void SystemToolbox::GetMacAddresses(std::set<std::string>& target)
+  {
+    target.clear();
+
+    SocketRaii socket;
+    
+    if (socket.GetDescriptor() != 1)
+    {
+      NetworkInterfaces interfaces;
+
+      while (!interfaces.IsDone())
+      {
+#if (defined(__APPLE__) && defined(__MACH__)) || defined(__FreeBSD_kernel__) || defined(__FreeBSD__)
+        int mib[6];
+        mib[0] = CTL_NET;
+        mib[1] = AF_ROUTE;
+        mib[2] = 0;
+        mib[3] = AF_LINK;
+        mib[4] = NET_RT_IFLIST;
+        mib[5] = interfaces.GetCurrentIndex();
+
+        size_t len;
+        if (sysctl(mib, 6, NULL, &len, NULL, 0) == 0 &&
+            len > 0)
+        {
+          std::string tmp;
+          tmp.resize(len);
+          if (sysctl(mib, 6, &tmp[0], &len, NULL, 0) == 0)
+          {
+            struct if_msghdr* ifm = reinterpret_cast<struct if_msghdr*>(&tmp[0]);
+            struct sockaddr_dl* sdl = reinterpret_cast<struct sockaddr_dl*>(ifm + 1);
+
+            if (sdl->sdl_type == IFT_ETHER)  // Only consider Ethernet interfaces
+            {
+              const unsigned char* mac = reinterpret_cast<const unsigned char*>(LLADDR(sdl));
+              char tmp[32];
+              sprintf(tmp, "%02x:%02x:%02x:%02x:%02x:%02x",
+                      mac[0], mac[1], mac[2], mac[3], mac[4], mac[5]);
+              target.insert(tmp);
+            }
+          }
+        }
+
+#else
+        struct ifreq ifr;
+        strcpy(ifr.ifr_name, interfaces.GetCurrentName());
+          
+        if (ioctl(socket.GetDescriptor(), SIOCGIFFLAGS, &ifr) == 0 &&
+            !(ifr.ifr_flags & IFF_LOOPBACK) && // ignore loopback interface
+            ioctl(socket.GetDescriptor(), SIOCGIFHWADDR, &ifr) == 0)
+        {
+          const unsigned char* mac = reinterpret_cast<const unsigned char*>(ifr.ifr_hwaddr.sa_data);
+            
+          char tmp[32];
+          sprintf(tmp, "%02x:%02x:%02x:%02x:%02x:%02x",
+                  mac[0], mac[1], mac[2], mac[3], mac[4], mac[5]);
+          target.insert(tmp);
+        }
+#endif
+        
+        interfaces.Next();
+      }
+    }
+  }
+
+#endif
 }
