@@ -2,7 +2,7 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017-2020 Osimis S.A., Belgium
+ * Copyright (C) 2017-2021 Osimis S.A., Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
@@ -52,16 +52,28 @@ namespace Orthanc
 
     static void ReadFile(std::string& content,
                          const std::string& path,
-                         bool log = true);
+                         bool log);
+
+    static void ReadFile(std::string& content,
+                         const std::string& path);
 
     static bool ReadHeader(std::string& header,
                            const std::string& path,
                            size_t headerSize);
 
+    static void WriteFile(const void* content,
+                          size_t size,
+                          const std::string& path,
+                          bool callFsync);
+
     static void WriteFile(const void* content,
                           size_t size,
                           const std::string& path);
 
+    static void WriteFile(const std::string& content,
+                          const std::string& path,
+                          bool callFsync);
+
     static void WriteFile(const std::string& content,
                           const std::string& path);
 
@@ -101,5 +113,13 @@ namespace Orthanc
 
     static std::string InterpretRelativePath(const std::string& baseDirectory,
                                              const std::string& relativePath);
+
+    static void ReadFileRange(std::string& content,                              
+                              const std::string& path,
+                              uint64_t start,  // Inclusive
+                              uint64_t end,    // Exclusive
+                              bool throwIfOverflow);
+
+    static void GetMacAddresses(std::set<std::string>& target);
   };
 }
