@@ -2,7 +2,7 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017-2020 Osimis S.A., Belgium
+ * Copyright (C) 2017-2021 Osimis S.A., Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
@@ -34,15 +34,18 @@
 #include <gtest/gtest.h>
 
 #include "../Sources/DicomFormat/DicomTag.h"
-#include "../Sources/FileBuffer.h"
 #include "../Sources/HttpServer/HttpToolbox.h"
 #include "../Sources/Logging.h"
-#include "../Sources/MetricsRegistry.h"
 #include "../Sources/OrthancException.h"
-#include "../Sources/SystemToolbox.h"
-#include "../Sources/TemporaryFile.h"
 #include "../Sources/Toolbox.h"
 
+#if ORTHANC_SANDBOXED != 1
+#  include "../Sources/FileBuffer.h"
+#  include "../Sources/MetricsRegistry.h"
+#  include "../Sources/SystemToolbox.h"
+#  include "../Sources/TemporaryFile.h"
+#endif
+
 #include <ctype.h>
 
 
@@ -100,10 +103,10 @@ TEST(Toolbox, IsSHA1)
 
 TEST(ParseGetArguments, Basic)
 {
-  IHttpHandler::GetArguments b;
+  HttpToolbox::GetArguments b;
   HttpToolbox::ParseGetArguments(b, "aaa=baaa&bb=a&aa=c");
 
-  IHttpHandler::Arguments a;
+  HttpToolbox::Arguments a;
   HttpToolbox::CompileGetArguments(a, b);
 
   ASSERT_EQ(3u, a.size());
@@ -112,12 +115,13 @@ TEST(ParseGetArguments, Basic)
   ASSERT_EQ(a["aa"], "c");
 }
 
+
 TEST(ParseGetArguments, BasicEmpty)
 {
-  IHttpHandler::GetArguments b;
+  HttpToolbox::GetArguments b;
   HttpToolbox::ParseGetArguments(b, "aaa&bb=aa&aa");
 
-  IHttpHandler::Arguments a;
+  HttpToolbox::Arguments a;
   HttpToolbox::CompileGetArguments(a, b);
 
   ASSERT_EQ(3u, a.size());
@@ -126,37 +130,40 @@ TEST(ParseGetArguments, BasicEmpty)
   ASSERT_EQ(a["aa"], "");
 }
 
+
 TEST(ParseGetArguments, Single)
 {
-  IHttpHandler::GetArguments b;
+  HttpToolbox::GetArguments b;
   HttpToolbox::ParseGetArguments(b, "aaa=baaa");
 
-  IHttpHandler::Arguments a;
+  HttpToolbox::Arguments a;
   HttpToolbox::CompileGetArguments(a, b);
 
   ASSERT_EQ(1u, a.size());
   ASSERT_EQ(a["aaa"], "baaa");
 }
 
+
 TEST(ParseGetArguments, SingleEmpty)
 {
-  IHttpHandler::GetArguments b;
+  HttpToolbox::GetArguments b;
   HttpToolbox::ParseGetArguments(b, "aaa");
 
-  IHttpHandler::Arguments a;
+  HttpToolbox::Arguments a;
   HttpToolbox::CompileGetArguments(a, b);
 
   ASSERT_EQ(1u, a.size());
   ASSERT_EQ(a["aaa"], "");
 }
 
+
 TEST(ParseGetQuery, Test1)
 {
   UriComponents uri;
-  IHttpHandler::GetArguments b;
+  HttpToolbox::GetArguments b;
   HttpToolbox::ParseGetQuery(uri, b, "/instances/test/world?aaa=baaa&bb=a&aa=c");
 
-  IHttpHandler::Arguments a;
+  HttpToolbox::Arguments a;
   HttpToolbox::CompileGetArguments(a, b);
 
   ASSERT_EQ(3u, uri.size());
@@ -169,13 +176,14 @@ TEST(ParseGetQuery, Test1)
   ASSERT_EQ(a["aa"], "c");
 }
 
+
 TEST(ParseGetQuery, Test2)
 {
   UriComponents uri;
-  IHttpHandler::GetArguments b;
+  HttpToolbox::GetArguments b;
   HttpToolbox::ParseGetQuery(uri, b, "/instances/test/world");
 
-  IHttpHandler::Arguments a;
+  HttpToolbox::Arguments a;
   HttpToolbox::CompileGetArguments(a, b);
 
   ASSERT_EQ(3u, uri.size());
@@ -185,6 +193,7 @@ TEST(ParseGetQuery, Test2)
   ASSERT_EQ(0u, a.size());
 }
 
+
 TEST(Uri, SplitUriComponents)
 {
   UriComponents c, d;
@@ -298,6 +307,8 @@ TEST(Uri, Child)
   ASSERT_TRUE(Toolbox::IsChildUri(c5, c5));
 }
 
+
+#if ORTHANC_SANDBOXED != 1
 TEST(Uri, AutodetectMimeType)
 {
   ASSERT_EQ(MimeType_Binary, SystemToolbox::AutodetectMimeType("../NOTES"));
@@ -351,6 +362,8 @@ TEST(Uri, AutodetectMimeType)
   ASSERT_STREQ("text/css", EnumerationToString(SystemToolbox::AutodetectMimeType(".css")));
   ASSERT_STREQ("text/html", EnumerationToString(SystemToolbox::AutodetectMimeType(".html")));
 }
+#endif
+
 
 TEST(Toolbox, ComputeMD5)
 {
@@ -374,11 +387,13 @@ TEST(Toolbox, ComputeSHA1)
   ASSERT_EQ("da39a3ee-5e6b4b0d-3255bfef-95601890-afd80709", s);
 }
 
+#if ORTHANC_SANDBOXED != 1
 TEST(Toolbox, PathToExecutable)
 {
   printf("[%s]\n", SystemToolbox::GetPathToExecutable().c_str());
   printf("[%s]\n", SystemToolbox::GetDirectoryOfExecutable().c_str());
 }
+#endif
 
 TEST(Toolbox, StripSpaces)
 {
@@ -386,6 +401,27 @@ TEST(Toolbox, StripSpaces)
   ASSERT_EQ("coucou", Toolbox::StripSpaces("    coucou   \t  \r   \n  "));
   ASSERT_EQ("cou   cou", Toolbox::StripSpaces("    cou   cou    \n  "));
   ASSERT_EQ("c", Toolbox::StripSpaces("    \n\t c\r    \n  "));
+
+  std::string s = "\"  abd \"";
+  Toolbox::RemoveSurroundingQuotes(s); ASSERT_EQ("  abd ", s);
+
+  s = "  \"  abd \"  ";
+  Toolbox::RemoveSurroundingQuotes(s); ASSERT_EQ("  \"  abd \"  ", s);
+
+  s = Toolbox::StripSpaces(s);
+  Toolbox::RemoveSurroundingQuotes(s); ASSERT_EQ("  abd ", s);
+
+  s = "\"";
+  Toolbox::RemoveSurroundingQuotes(s); ASSERT_EQ("", s);  
+
+  s = "\"\"";
+  Toolbox::RemoveSurroundingQuotes(s); ASSERT_EQ("", s);  
+
+  s = "\"_\"";
+  Toolbox::RemoveSurroundingQuotes(s); ASSERT_EQ("_", s);
+
+  s = "\"\"\"";
+  Toolbox::RemoveSurroundingQuotes(s); ASSERT_EQ("\"", s);
 }
 
 TEST(Toolbox, Case)
@@ -567,6 +603,7 @@ TEST(Toolbox, AbsoluteDirectory)
 #endif
 
 
+#if ORTHANC_SANDBOXED != 1
 TEST(Toolbox, WriteFile)
 {
   std::string path;
@@ -603,9 +640,20 @@ TEST(Toolbox, WriteFile)
 
   std::string u;
   ASSERT_THROW(SystemToolbox::ReadFile(u, path.c_str()), OrthancException);
+
+  {
+    TemporaryFile tmp;
+    std::string s = "Hello";
+    SystemToolbox::WriteFile(s, tmp.GetPath(), true /* call fsync() */);
+    std::string t;
+    SystemToolbox::ReadFile(t, tmp.GetPath());
+    ASSERT_EQ(s, t);
+  }
 }
+#endif
 
 
+#if ORTHANC_SANDBOXED != 1
 TEST(Toolbox, FileBuffer)
 {
   FileBuffer f;
@@ -619,6 +667,7 @@ TEST(Toolbox, FileBuffer)
 
   ASSERT_THROW(f.Append("d", 1), OrthancException);  // File is closed
 }
+#endif
 
 
 TEST(Toolbox, Wildcard)
@@ -759,6 +808,23 @@ TEST(Toolbox, Enumerations)
   ASSERT_FALSE(IsResourceLevelAboveOrEqual(ResourceType_Instance, ResourceType_Study));
   ASSERT_FALSE(IsResourceLevelAboveOrEqual(ResourceType_Instance, ResourceType_Series));
   ASSERT_TRUE(IsResourceLevelAboveOrEqual(ResourceType_Instance, ResourceType_Instance));
+
+  ASSERT_STREQ("Patients", GetResourceTypeText(ResourceType_Patient, true /* plural */, true /* upper case */));
+  ASSERT_STREQ("patients", GetResourceTypeText(ResourceType_Patient, true, false));
+  ASSERT_STREQ("Patient", GetResourceTypeText(ResourceType_Patient, false, true));
+  ASSERT_STREQ("patient", GetResourceTypeText(ResourceType_Patient, false, false));
+  ASSERT_STREQ("Studies", GetResourceTypeText(ResourceType_Study, true, true));
+  ASSERT_STREQ("studies", GetResourceTypeText(ResourceType_Study, true, false));
+  ASSERT_STREQ("Study", GetResourceTypeText(ResourceType_Study, false, true));
+  ASSERT_STREQ("study", GetResourceTypeText(ResourceType_Study, false, false));
+  ASSERT_STREQ("Series", GetResourceTypeText(ResourceType_Series, true, true));
+  ASSERT_STREQ("series", GetResourceTypeText(ResourceType_Series, true, false));
+  ASSERT_STREQ("Series", GetResourceTypeText(ResourceType_Series, false, true));
+  ASSERT_STREQ("series", GetResourceTypeText(ResourceType_Series, false, false));
+  ASSERT_STREQ("Instances", GetResourceTypeText(ResourceType_Instance, true, true));
+  ASSERT_STREQ("instances", GetResourceTypeText(ResourceType_Instance, true, false));
+  ASSERT_STREQ("Instance", GetResourceTypeText(ResourceType_Instance, false, true));
+  ASSERT_STREQ("instance", GetResourceTypeText(ResourceType_Instance, false, false));
 }
 
 
@@ -811,8 +877,15 @@ TEST(Toolbox, Endianness)
   ASSERT_EQ(Endianness_Little, Toolbox::DetectEndianness());
 #  endif
 
+  
+  /**
+   * WebAssembly is always little-endian.
+   **/
+  
+#elif defined(__EMSCRIPTEN__)
+  ASSERT_EQ(Endianness_Little, Toolbox::DetectEndianness());
 #else
-#error Support your platform here
+#  error Support your platform here
 #endif
 }
 
@@ -998,6 +1071,7 @@ TEST(Toolbox, EndiannessConversions64)
 }
 
 
+#if ORTHANC_SANDBOXED != 1
 TEST(Toolbox, Now)
 {
   LOG(WARNING) << "Local time: " << SystemToolbox::GetNowIsoString(false);
@@ -1010,7 +1084,7 @@ TEST(Toolbox, Now)
   SystemToolbox::GetNowDicom(date, time, true);
   LOG(WARNING) << "Universal DICOM time: [" << date << "] [" << time << "]";
 }
-
+#endif
 
 
 #if ORTHANC_ENABLE_PUGIXML == 1
@@ -1031,7 +1105,7 @@ TEST(Toolbox, Xml)
 #endif
 
 
-#if !defined(_WIN32)
+#if !defined(_WIN32) && (ORTHANC_SANDBOXED != 1)
 TEST(Toolbox, ExecuteSystemCommand)
 {
   std::vector<std::string> args(2);
@@ -1077,7 +1151,7 @@ TEST(Toolbox, UriEncode)
   Toolbox::UriEncode(s, t); 
   ASSERT_EQ(t, s);
 
-  Toolbox::UriEncode(s, "!#$&'()*+,/:;=?@[]"); ASSERT_EQ("%21%23%24%26%27%28%29%2A%2B%2C%2F%3A%3B%3D%3F%40%5B%5D", s);  
+  Toolbox::UriEncode(s, "!#$&'()*+,/:;=?@[]"); ASSERT_EQ("%21%23%24%26%27%28%29%2A%2B%2C/%3A%3B%3D%3F%40%5B%5D", s);  
   Toolbox::UriEncode(s, "%"); ASSERT_EQ("%25", s);
 
   // Encode characters from UTF-8. This is the test string from the
@@ -1191,6 +1265,7 @@ TEST(Toolbox, LinesIterator)
 }
 
 
+#if ORTHANC_SANDBOXED != 1
 TEST(Toolbox, SubstituteVariables)
 {
   std::map<std::string, std::string> env;
@@ -1219,8 +1294,10 @@ TEST(Toolbox, SubstituteVariables)
   ASSERT_EQ("AhelloB",
             Toolbox::SubstituteVariables("A${PATH}B", env));
 }
+#endif
 
 
+#if ORTHANC_SANDBOXED != 1
 TEST(MetricsRegistry, Basic)
 {
   {
@@ -1333,3 +1410,52 @@ TEST(MetricsRegistry, Basic)
     ASSERT_EQ(MetricsType_MinOver10Seconds, m.GetMetricsType("b"));
   }
 }
+#endif
+
+
+#if ORTHANC_SANDBOXED != 1
+TEST(Toolbox, ReadFileRange)
+{
+  TemporaryFile tmp;
+  std::string s;
+
+  tmp.Write("");
+  tmp.Read(s);                     ASSERT_TRUE(s.empty());
+  tmp.ReadRange(s, 0, 0, true);    ASSERT_TRUE(s.empty());
+  tmp.ReadRange(s, 0, 10, false);  ASSERT_TRUE(s.empty());
+  
+  ASSERT_THROW(tmp.ReadRange(s, 0, 1, true), OrthancException);
+  
+  tmp.Write("Hello");
+  tmp.Read(s);                     ASSERT_EQ("Hello", s);
+  tmp.ReadRange(s, 0, 5, true);    ASSERT_EQ("Hello", s);
+  tmp.ReadRange(s, 0, 1, true);    ASSERT_EQ("H", s);
+  tmp.ReadRange(s, 1, 2, true);    ASSERT_EQ("e", s);
+  tmp.ReadRange(s, 2, 3, true);    ASSERT_EQ("l", s);
+  tmp.ReadRange(s, 3, 4, true);    ASSERT_EQ("l", s);
+  tmp.ReadRange(s, 4, 5, true);    ASSERT_EQ("o", s);
+  tmp.ReadRange(s, 2, 5, true);    ASSERT_EQ("llo", s);
+  tmp.ReadRange(s, 2, 50, false);  ASSERT_EQ("llo", s);
+  tmp.ReadRange(s, 2, 2, false);   ASSERT_TRUE(s.empty());
+  tmp.ReadRange(s, 10, 50, false); ASSERT_TRUE(s.empty());
+  
+  ASSERT_THROW(tmp.ReadRange(s, 5, 10, true), OrthancException);
+  ASSERT_THROW(tmp.ReadRange(s, 10, 50, true), OrthancException);
+  ASSERT_THROW(tmp.ReadRange(s, 50, 10, true), OrthancException);
+  ASSERT_THROW(tmp.ReadRange(s, 2, 1, true), OrthancException);
+}
+#endif
+
+
+#if ORTHANC_SANDBOXED != 1
+TEST(Toolbox, GetMacAddressess)
+{
+  std::set<std::string> mac;
+  SystemToolbox::GetMacAddresses(mac);
+
+  for (std::set<std::string>::const_iterator it = mac.begin(); it != mac.end(); ++it)
+  {
+    printf("MAC address: [%s]\n", it->c_str());
+  }
+}
+#endif
