@@ -2,7 +2,7 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017-2020 Osimis S.A., Belgium
+ * Copyright (C) 2017-2021 Osimis S.A., Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -61,10 +61,10 @@ namespace Orthanc
 #include "../../../OrthancFramework/Sources/HttpServer/IHttpHandler.h"
 #include "../../../OrthancFramework/Sources/HttpServer/IIncomingHttpRequestFilter.h"
 #include "../../../OrthancFramework/Sources/JobsEngine/IJob.h"
+#include "../../Sources/Database/IDatabaseWrapper.h"
 #include "../../Sources/IDicomImageDecoder.h"
 #include "../../Sources/IServerListener.h"
 #include "../../Sources/ServerJobs/IStorageCommitmentFactory.h"
-#include "OrthancPluginDatabase.h"
 #include "PluginsManager.h"
 
 #include <list>
@@ -80,7 +80,6 @@ namespace Orthanc
     public IServerListener,
     public IWorklistRequestHandlerFactory,
     public IDicomImageDecoder,
-    public IIncomingHttpRequestFilter,
     public IFindRequestHandlerFactory,
     public IMoveRequestHandlerFactory,
     public IStorageCommitmentFactory,
@@ -109,8 +108,8 @@ namespace Orthanc
     bool HandleChunkedGetDelete(HttpOutput& output,
                                 HttpMethod method,
                                 const UriComponents& uri,
-                                const Arguments& headers,
-                                const GetArguments& getArguments);
+                                const HttpToolbox::Arguments& headers,
+                                const HttpToolbox::GetArguments& getArguments);
 
     void RegisterOnStoredInstanceCallback(const void* parameters);
 
@@ -197,6 +196,8 @@ namespace Orthanc
 
     void ChunkedHttpClient(const void* parameters);
 
+    void CallRestApi(const void* parameters);
+
     void CallPeerApi(const void* parameters);
   
     void GetFontInfo(const void* parameters);
@@ -208,8 +209,8 @@ namespace Orthanc
     void ApplyDicomToJson(_OrthancPluginService service,
                           const void* parameters);
 
-    void ApplyCreateDicom(_OrthancPluginService service,
-                          const void* parameters);
+    void ApplyCreateDicom(const _OrthancPluginCreateDicom& parameters,
+                          const char* privateCreatorC);
 
     void ApplyCreateImage(_OrthancPluginService service,
                           const void* parameters);
@@ -246,7 +247,7 @@ namespace Orthanc
                                  bool allowNewSopInstanceUid) ORTHANC_OVERRIDE;
     
   public:
-    OrthancPlugins();
+    explicit OrthancPlugins(const std::string& databaseServerIdentifier);
 
     virtual ~OrthancPlugins();
 
@@ -260,8 +261,8 @@ namespace Orthanc
                         const char* username,
                         HttpMethod method,
                         const UriComponents& uri,
-                        const Arguments& headers,
-                        const GetArguments& getArguments,
+                        const HttpToolbox::Arguments& headers,
+                        const HttpToolbox::GetArguments& getArguments,
                         const void* bodyData,
                         size_t bodySize) ORTHANC_OVERRIDE;
 
@@ -348,12 +349,12 @@ namespace Orthanc
                                   size_t size,
                                   unsigned int frame) ORTHANC_OVERRIDE;
 
-    virtual bool IsAllowed(HttpMethod method,
-                           const char* uri,
-                           const char* ip,
-                           const char* username,
-                           const IHttpHandler::Arguments& httpHeaders,
-                           const IHttpHandler::GetArguments& getArguments) ORTHANC_OVERRIDE;
+    bool IsAllowed(HttpMethod method,
+                   const char* uri,
+                   const char* ip,
+                   const char* username,
+                   const HttpToolbox::Arguments& httpHeaders,
+                   const HttpToolbox::GetArguments& getArguments);
 
     bool HasFindHandler();
 
@@ -375,7 +376,7 @@ namespace Orthanc
                                             const char* username,
                                             HttpMethod method,
                                             const UriComponents& uri,
-                                            const Arguments& headers) ORTHANC_OVERRIDE;
+                                            const HttpToolbox::Arguments& headers) ORTHANC_OVERRIDE;
 
     // New in Orthanc 1.6.0
     IStorageCommitmentFactory::ILookupHandler* CreateStorageCommitment(
@@ -385,6 +386,11 @@ namespace Orthanc
       const std::vector<std::string>& sopInstanceUids,
       const std::string& remoteAet,
       const std::string& calledAet) ORTHANC_OVERRIDE;
+
+    // New in Orthanc 1.8.1 (cf. "OrthancPluginGenerateRestApiAuthorizationToken()")
+    bool IsValidAuthorizationToken(const std::string& token) const;
+
+    unsigned int GetMaxDatabaseRetries() const;
   };
 }
 
