@@ -43,6 +43,8 @@
 #include <boost/thread/shared_mutex.hpp>
 #include <boost/thread/lock_types.hpp>
 
+class DcmDataset;
+
 namespace Orthanc
 {
   class DicomMap;
@@ -249,12 +251,20 @@ namespace Orthanc
 
     void GetAcceptedTransferSyntaxes(std::set<DicomTransferSyntax>& target) const;
 
+    std::string GetDatabaseServerIdentifier() const;
+
     static void DefaultExtractDicomSummary(DicomMap& target,
                                            const ParsedDicomFile& dicom);
+
+    static void DefaultExtractDicomSummary(DicomMap& target,
+                                           DcmDataset& dicom);
     
     static void DefaultDicomDatasetToJson(Json::Value& target,
                                           const ParsedDicomFile& dicom);
     
+    static void DefaultDicomDatasetToJson(Json::Value& target,
+                                          DcmDataset& dicom);
+    
     static void DefaultDicomDatasetToJson(Json::Value& target,
                                           const ParsedDicomFile& dicom,
                                           const std::set<DicomTag>& ignoreTagLength);
