@@ -36,6 +36,7 @@
 
 #include "../../OrthancFramework/Sources/Cache/SharedArchive.h"
 #include "../../OrthancFramework/Sources/DicomFormat/DicomElement.h"
+#include "../../OrthancFramework/Sources/DicomFormat/DicomStreamReader.h"
 #include "../../OrthancFramework/Sources/DicomParsing/DcmtkTranscoder.h"
 #include "../../OrthancFramework/Sources/DicomParsing/DicomModification.h"
 #include "../../OrthancFramework/Sources/DicomParsing/FromDcmtkBridge.h"
@@ -214,7 +215,7 @@ namespace Orthanc
     if (loadJobsFromDatabase)
     {
       std::string serialized;
-      if (index_.LookupGlobalProperty(serialized, GlobalProperty_JobsRegistry))
+      if (index_.LookupGlobalProperty(serialized, GlobalProperty_JobsRegistry, false /* not shared */))
       {
         LOG(WARNING) << "Reloading the jobs from the last execution of Orthanc";
 
@@ -260,7 +261,7 @@ namespace Orthanc
         std::string serialized;
         Toolbox::WriteFastJson(serialized, value);
 
-        index_.SetGlobalProperty(GlobalProperty_JobsRegistry, serialized);
+        index_.SetGlobalProperty(GlobalProperty_JobsRegistry, false /* not shared */, serialized);
       }
       catch (OrthancException& e)
       {
@@ -509,17 +510,32 @@ namespace Orthanc
 
       default:
         throw OrthancException(ErrorCode_ParameterOutOfRange);
-    }    
+    }
+
+    bool hasPixelDataOffset;
+    uint64_t pixelDataOffset;
+    hasPixelDataOffset = DicomStreamReader::LookupPixelDataOffset(
+      pixelDataOffset, dicom.GetBufferData(), dicom.GetBufferSize());
+
+    DicomTransferSyntax transferSyntax;
+    bool hasTransferSyntax = dicom.LookupTransferSyntax(transferSyntax);
     
+    DicomMap summary;
+    dicom.GetSummary(summary);
+
     try
     {
       MetricsRegistry::Timer timer(GetMetricsRegistry(), "orthanc_store_dicom_duration_ms");
       StorageAccessor accessor(area_, GetMetricsRegistry());
 
-      resultPublicId = dicom.GetHasher().HashInstance();
+      DicomInstanceHasher hasher(summary);
+      resultPublicId = hasher.HashInstance();
 
+      Json::Value dicomAsJson;
+      dicom.GetDicomAsJson(dicomAsJson);
+      
       Json::Value simplifiedTags;
-      Toolbox::SimplifyDicomAsJson(simplifiedTags, dicom.GetJson(), DicomToJsonFormat_Human);
+      Toolbox::SimplifyDicomAsJson(simplifiedTags, dicomAsJson, DicomToJsonFormat_Human);
 
       // Test if the instance must be filtered out
       bool accepted = true;
@@ -563,32 +579,43 @@ namespace Orthanc
 
       FileInfo dicomInfo = accessor.Write(dicom.GetBufferData(), dicom.GetBufferSize(), 
                                           FileContentType_Dicom, compression, storeMD5_);
-      FileInfo jsonInfo = accessor.Write(dicom.GetJson().toStyledString(), 
-                                         FileContentType_DicomAsJson, compression, storeMD5_);
 
       ServerIndex::Attachments attachments;
       attachments.push_back(dicomInfo);
-      attachments.push_back(jsonInfo);
+
+      FileInfo dicomUntilPixelData;
+      if (hasPixelDataOffset &&
+          (!area_.HasReadRange() ||
+           compressionEnabled_))
+      {
+        dicomUntilPixelData = accessor.Write(dicom.GetBufferData(), pixelDataOffset, 
+                                             FileContentType_DicomUntilPixelData, compression, storeMD5_);
+        attachments.push_back(dicomUntilPixelData);
+      }
 
       typedef std::map<MetadataType, std::string>  InstanceMetadata;
       InstanceMetadata  instanceMetadata;
       StoreStatus status = index_.Store(
-        instanceMetadata, dicom, attachments, overwrite);
+        instanceMetadata, summary, attachments, dicom.GetMetadata(), dicom.GetOrigin(), overwrite,
+        hasTransferSyntax, transferSyntax, hasPixelDataOffset, pixelDataOffset);
 
       // Only keep the metadata for the "instance" level
-      dicom.GetMetadata().clear();
+      dicom.ClearMetadata();
 
       for (InstanceMetadata::const_iterator it = instanceMetadata.begin();
            it != instanceMetadata.end(); ++it)
       {
-        dicom.GetMetadata().insert(std::make_pair(std::make_pair(ResourceType_Instance, it->first),
-                                                  it->second));
+        dicom.AddMetadata(ResourceType_Instance, it->first, it->second);
       }
             
       if (status != StoreStatus_Success)
       {
         accessor.Remove(dicomInfo);
-        accessor.Remove(jsonInfo);
+
+        if (dicomUntilPixelData.IsValid())
+        {
+          accessor.Remove(dicomUntilPixelData);
+        }
       }
 
       switch (status)
@@ -636,9 +663,9 @@ namespace Orthanc
     {
       if (e.GetErrorCode() == ErrorCode_InexistentTag)
       {
-        dicom.GetSummary().LogMissingTagsForStore();
+        summary.LogMissingTagsForStore();
       }
-
+      
       throw;
     }
   }
@@ -660,8 +687,7 @@ namespace Orthanc
       bool transcode = false;
 
       DicomTransferSyntax sourceSyntax;
-      if (!FromDcmtkBridge::LookupOrthancTransferSyntax(
-            sourceSyntax, dicom.GetParsedDicomFile().GetDcmtkObject()) ||
+      if (!dicom.LookupTransferSyntax(sourceSyntax) ||
           sourceSyntax == ingestTransferSyntax_)
       {
         // Don't transcode if the incoming DICOM is already in the proper transfer syntax
@@ -704,11 +730,10 @@ namespace Orthanc
         {
           std::unique_ptr<ParsedDicomFile> tmp(transcoded.ReleaseAsParsedDicomFile());
 
-          DicomInstanceToStore toStore;
-          toStore.SetParsedDicomFile(*tmp);
-          toStore.SetOrigin(dicom.GetOrigin());
+          std::unique_ptr<DicomInstanceToStore> toStore(DicomInstanceToStore::CreateFromParsedDicomFile(*tmp));
+          toStore->SetOrigin(dicom.GetOrigin());
 
-          StoreStatus ok = StoreAfterTranscoding(resultPublicId, toStore, mode);
+          StoreStatus ok = StoreAfterTranscoding(resultPublicId, *toStore, mode);
           assert(resultPublicId == tmp->GetHasher().HashInstance());
 
           return ok;
@@ -728,13 +753,16 @@ namespace Orthanc
                                        FileContentType content)
   {
     FileInfo attachment;
-    if (!index_.LookupAttachment(attachment, resourceId, content))
+    int64_t revision;
+    if (!index_.LookupAttachment(attachment, revision, resourceId, content))
     {
       throw OrthancException(ErrorCode_UnknownResource);
     }
-
-    StorageAccessor accessor(area_, GetMetricsRegistry());
-    accessor.AnswerFile(output, attachment, GetFileContentMime(content));
+    else
+    {
+      StorageAccessor accessor(area_, GetMetricsRegistry());
+      accessor.AnswerFile(output, attachment, GetFileContentMime(content));
+    }
   }
 
 
@@ -748,7 +776,8 @@ namespace Orthanc
               << compression; 
 
     FileInfo attachment;
-    if (!index_.LookupAttachment(attachment, resourceId, attachmentType))
+    int64_t revision;
+    if (!index_.LookupAttachment(attachment, revision, resourceId, attachmentType))
     {
       throw OrthancException(ErrorCode_UnknownResource);
     }
@@ -769,7 +798,8 @@ namespace Orthanc
 
     try
     {
-      StoreStatus status = index_.AddAttachment(modified, resourceId);
+      int64_t newRevision;  // ignored
+      StoreStatus status = index_.AddAttachment(newRevision, modified, resourceId, true, revision);
       if (status != StoreStatus_Success)
       {
         accessor.Remove(modified);
@@ -784,92 +814,253 @@ namespace Orthanc
   }
 
 
-  void ServerContext::ReadDicomAsJsonInternal(std::string& result,
-                                              const std::string& instancePublicId)
+  static void InjectEmptyPixelData(Json::Value& dicomAsJson)
   {
-    FileInfo attachment;
-    if (index_.LookupAttachment(attachment, instancePublicId, FileContentType_DicomAsJson))
-    {
-      ReadAttachment(result, attachment);
-    }
-    else
-    {
-      // The "DICOM as JSON" summary is not available from the Orthanc
-      // store (most probably deleted), reconstruct it from the DICOM file
-      std::string dicom;
-      ReadDicom(dicom, instancePublicId);
+    // This is for backward compatibility with Orthanc <= 1.9.0
+    Json::Value pixelData = Json::objectValue;
+    pixelData["Name"] = "PixelData";
+    pixelData["Type"] = "Null";
+    pixelData["Value"] = Json::nullValue;
 
-      LOG(INFO) << "Reconstructing the missing DICOM-as-JSON summary for instance: "
-                << instancePublicId;
-    
-      ParsedDicomFile parsed(dicom);
-
-      Json::Value summary;
-      OrthancConfiguration::DefaultDicomDatasetToJson(summary, parsed);
-
-      result = summary.toStyledString();
-
-      if (!AddAttachment(instancePublicId, FileContentType_DicomAsJson,
-                         result.c_str(), result.size()))
-      {
-        throw OrthancException(ErrorCode_InternalError,
-                               "Cannot associate the DICOM-as-JSON summary to instance: " + instancePublicId);
-      }
-    }
+    dicomAsJson["7fe0,0010"] = pixelData;
   }
 
-
-  void ServerContext::ReadDicomAsJson(std::string& result,
+  
+  void ServerContext::ReadDicomAsJson(Json::Value& result,
                                       const std::string& instancePublicId,
                                       const std::set<DicomTag>& ignoreTagLength)
   {
-    if (ignoreTagLength.empty())
+    /**
+     * CASE 1: The DICOM file, truncated at pixel data, is available
+     * as an attachment (it was created either because the storage
+     * area does not support range reads, or it "StorageCompression"
+     * is enabled). Simply return this attachment.
+     **/
+    
+    FileInfo attachment;
+    int64_t revision;  // Ignored
+
+    if (index_.LookupAttachment(attachment, revision, instancePublicId, FileContentType_DicomUntilPixelData))
     {
-      ReadDicomAsJsonInternal(result, instancePublicId);
+      std::string dicom;
+
+      {
+        StorageAccessor accessor(area_, GetMetricsRegistry());
+        accessor.Read(dicom, attachment);
+      }
+
+      ParsedDicomFile parsed(dicom);
+      OrthancConfiguration::DefaultDicomDatasetToJson(result, parsed, ignoreTagLength);
+      InjectEmptyPixelData(result);
     }
     else
     {
-      Json::Value tmp;
-      ReadDicomAsJson(tmp, instancePublicId, ignoreTagLength);
-      result = tmp.toStyledString();
+      /**
+       * The truncated DICOM file is not stored as a standalone
+       * attachment. Lookup whether the pixel data offset has already
+       * been computed for this instance.
+       **/
+    
+      bool hasPixelDataOffset;
+      uint64_t pixelDataOffset = 0;  // dummy initialization
+
+      {
+        std::string s;
+        if (index_.LookupMetadata(s, revision, instancePublicId, ResourceType_Instance,
+                                  MetadataType_Instance_PixelDataOffset))
+        {
+          hasPixelDataOffset = false;
+
+          if (!s.empty())
+          {
+            try
+            {
+              pixelDataOffset = boost::lexical_cast<uint64_t>(s);
+              hasPixelDataOffset = true;
+            }
+            catch (boost::bad_lexical_cast&)
+            {
+            }
+          }
+
+          if (!hasPixelDataOffset)
+          {
+            LOG(ERROR) << "Metadata \"PixelDataOffset\" is corrupted for instance: " << instancePublicId;
+          }
+        }
+        else
+        {
+          // This instance was created by a version of Orthanc <= 1.9.0
+          hasPixelDataOffset = false;
+        }
+      }
+
+
+      if (hasPixelDataOffset &&
+          area_.HasReadRange() &&
+          index_.LookupAttachment(attachment, revision, instancePublicId, FileContentType_Dicom) &&
+          attachment.GetCompressionType() == CompressionType_None)
+      {
+        /**
+         * CASE 2: The pixel data offset is known, AND that a range read
+         * can be used to retrieve the truncated DICOM file. Note that
+         * this case cannot be used if "StorageCompression" option is
+         * "true".
+         **/
+      
+        std::unique_ptr<IMemoryBuffer> dicom;
+        {
+          MetricsRegistry::Timer timer(GetMetricsRegistry(), "orthanc_storage_read_range_duration_ms");
+          dicom.reset(area_.ReadRange(attachment.GetUuid(), FileContentType_Dicom, 0, pixelDataOffset));
+        }
+
+        if (dicom.get() == NULL)
+        {
+          throw OrthancException(ErrorCode_InternalError);
+        }
+        else
+        {
+          assert(dicom->GetSize() == pixelDataOffset);
+          ParsedDicomFile parsed(dicom->GetData(), dicom->GetSize());
+          OrthancConfiguration::DefaultDicomDatasetToJson(result, parsed, ignoreTagLength);
+          InjectEmptyPixelData(result);
+        }
+      }
+      else if (ignoreTagLength.empty() &&
+               index_.LookupAttachment(attachment, revision, instancePublicId, FileContentType_DicomAsJson))
+      {
+        /**
+         * CASE 3: This instance was created using Orthanc <=
+         * 1.9.0. Reuse the old "DICOM-as-JSON" attachment if available.
+         * This is for backward compatibility: A call to
+         * "/tools/invalidate-tags" or to one flavors of
+         * "/.../.../reconstruct" will disable this case.
+         **/
+      
+        std::string dicomAsJson;
+
+        {
+          StorageAccessor accessor(area_, GetMetricsRegistry());
+          accessor.Read(dicomAsJson, attachment);
+        }
+
+        if (!Toolbox::ReadJson(result, dicomAsJson))
+        {
+          throw OrthancException(ErrorCode_CorruptedFile,
+                                 "Corrupted DICOM-as-JSON attachment of instance: " + instancePublicId);
+        }
+      }
+      else
+      {
+        /**
+         * CASE 4: Neither the truncated DICOM file is accessible, nor
+         * the DICOM-as-JSON summary. We have to retrieve the full DICOM
+         * file from the storage area.
+         **/
+
+        std::string dicom;
+        ReadDicom(dicom, instancePublicId);
+
+        ParsedDicomFile parsed(dicom);
+        OrthancConfiguration::DefaultDicomDatasetToJson(result, parsed, ignoreTagLength);
+
+        if (!hasPixelDataOffset)
+        {
+          /**
+           * The pixel data offset was never computed for this
+           * instance, which indicates that it was created using
+           * Orthanc <= 1.9.0, or that calls to
+           * "LookupPixelDataOffset()" from earlier versions of
+           * Orthanc have failed. Try again this precomputation now
+           * for future calls.
+           **/
+          if (DicomStreamReader::LookupPixelDataOffset(pixelDataOffset, dicom) &&
+              pixelDataOffset < dicom.size())
+          {
+            index_.OverwriteMetadata(instancePublicId, MetadataType_Instance_PixelDataOffset,
+                                     boost::lexical_cast<std::string>(pixelDataOffset));
+
+            if (!area_.HasReadRange() ||
+                compressionEnabled_)
+            {
+              int64_t newRevision;
+              AddAttachment(newRevision, instancePublicId, FileContentType_DicomUntilPixelData,
+                            dicom.empty() ? NULL: dicom.c_str(), pixelDataOffset,
+                            false /* no old revision */, -1 /* dummy revision */);
+            }
+          }
+        }
+      }
     }
   }
 
 
   void ServerContext::ReadDicomAsJson(Json::Value& result,
-                                      const std::string& instancePublicId,
-                                      const std::set<DicomTag>& ignoreTagLength)
+                                      const std::string& instancePublicId)
   {
-    if (ignoreTagLength.empty())
-    {
-      std::string tmp;
-      ReadDicomAsJsonInternal(tmp, instancePublicId);
-
-      if (!Toolbox::ReadJson(result, tmp))
-      {
-        throw OrthancException(ErrorCode_CorruptedFile);
-      }
-    }
-    else
-    {
-      // The "DicomAsJson" attachment might have stored some tags as
-      // "too long". We are forced to re-parse the DICOM file.
-      std::string dicom;
-      ReadDicom(dicom, instancePublicId);
-
-      ParsedDicomFile parsed(dicom);
-      OrthancConfiguration::DefaultDicomDatasetToJson(result, parsed, ignoreTagLength);
-    }
+    std::set<DicomTag> ignoreTagLength;
+    ReadDicomAsJson(result, instancePublicId, ignoreTagLength);
   }
 
 
+  void ServerContext::ReadDicom(std::string& dicom,
+                                const std::string& instancePublicId)
+  {
+    int64_t revision;
+    ReadAttachment(dicom, revision, instancePublicId, FileContentType_Dicom, true /* uncompress */);
+  }
+    
+
+  bool ServerContext::ReadDicomUntilPixelData(std::string& dicom,
+                                              const std::string& instancePublicId)
+  {
+    if (!area_.HasReadRange())
+    {
+      return false;
+    }
+    
+    FileInfo attachment;
+    int64_t revision;  // Ignored
+    if (!index_.LookupAttachment(attachment, revision, instancePublicId, FileContentType_Dicom))
+    {
+      throw OrthancException(ErrorCode_InternalError,
+                             "Unable to read the DICOM file of instance " + instancePublicId);
+    }
+
+    std::string s;
+
+    if (attachment.GetCompressionType() == CompressionType_None &&
+        index_.LookupMetadata(s, revision, instancePublicId, ResourceType_Instance,
+                              MetadataType_Instance_PixelDataOffset) &&
+        !s.empty())
+    {
+      try
+      {
+        uint64_t pixelDataOffset = boost::lexical_cast<uint64_t>(s);
+
+        std::unique_ptr<IMemoryBuffer> buffer(
+          area_.ReadRange(attachment.GetUuid(), attachment.GetContentType(), 0, pixelDataOffset));
+        buffer->MoveToString(dicom);
+        return true;   // Success
+      }
+      catch (boost::bad_lexical_cast&)
+      {
+        LOG(ERROR) << "Metadata \"PixelDataOffset\" is corrupted for instance: " << instancePublicId;
+      }
+    }
+
+    return false;
+  }
+  
+
   void ServerContext::ReadAttachment(std::string& result,
+                                     int64_t& revision,
                                      const std::string& instancePublicId,
                                      FileContentType content,
                                      bool uncompressIfNeeded)
   {
     FileInfo attachment;
-    if (!index_.LookupAttachment(attachment, instancePublicId, content))
+    if (!index_.LookupAttachment(attachment, revision, instancePublicId, content))
     {
       throw OrthancException(ErrorCode_InternalError,
                              "Unable to read attachment " + EnumerationToString(content) +
@@ -878,29 +1069,23 @@ namespace Orthanc
 
     assert(attachment.GetContentType() == content);
 
-    if (uncompressIfNeeded)
     {
-      ReadAttachment(result, attachment);
-    }
-    else
-    {
-      // Do not uncompress the content of the storage area, return the
-      // raw data
       StorageAccessor accessor(area_, GetMetricsRegistry());
-      accessor.ReadRaw(result, attachment);
+
+      if (uncompressIfNeeded)
+      {
+        accessor.Read(result, attachment);
+      }
+      else
+      {
+        // Do not uncompress the content of the storage area, return the
+        // raw data
+        accessor.ReadRaw(result, attachment);
+      }
     }
   }
 
 
-  void ServerContext::ReadAttachment(std::string& result,
-                                     const FileInfo& attachment)
-  {
-    // This will decompress the attachment
-    StorageAccessor accessor(area_, GetMetricsRegistry());
-    accessor.Read(result, attachment);
-  }
-
-
   ServerContext::DicomCacheLocker::DicomCacheLocker(ServerContext& context,
                                                     const std::string& instancePublicId) :
     context_(context),
@@ -971,10 +1156,13 @@ namespace Orthanc
   }
 
 
-  bool ServerContext::AddAttachment(const std::string& resourceId,
+  bool ServerContext::AddAttachment(int64_t& newRevision,
+                                    const std::string& resourceId,
                                     FileContentType attachmentType,
                                     const void* data,
-                                    size_t size)
+                                    size_t size,
+                                    bool hasOldRevision,
+                                    int64_t oldRevision)
   {
     LOG(INFO) << "Adding attachment " << EnumerationToString(attachmentType) << " to resource " << resourceId;
     
@@ -984,7 +1172,7 @@ namespace Orthanc
     StorageAccessor accessor(area_, GetMetricsRegistry());
     FileInfo attachment = accessor.Write(data, size, attachmentType, compression, storeMD5_);
 
-    StoreStatus status = index_.AddAttachment(attachment, resourceId);
+    StoreStatus status = index_.AddAttachment(newRevision, attachment, resourceId, hasOldRevision, oldRevision);
     if (status != StoreStatus_Success)
     {
       accessor.Remove(attachment);
@@ -1469,7 +1657,8 @@ namespace Orthanc
     if (metadata == MetadataType_Instance_SopClassUid ||
         metadata == MetadataType_Instance_TransferSyntax)
     {
-      if (index_.LookupMetadata(target, publicId, level, metadata))
+      int64_t revision;  // Ignored
+      if (index_.LookupMetadata(target, revision, publicId, level, metadata))
       {
         return true;
       }
@@ -1511,7 +1700,7 @@ namespace Orthanc
           target = value->GetContent();
 
           // Store for reuse
-          index_.SetMetadata(publicId, metadata, target);
+          index_.OverwriteMetadata(publicId, metadata, target);
           return true;
         }
         else
@@ -1524,7 +1713,8 @@ namespace Orthanc
     else
     {
       // No backward
-      return index_.LookupMetadata(target, publicId, level, metadata);
+      int64_t revision;  // Ignored
+      return index_.LookupMetadata(target, revision, publicId, level, metadata);
     }
   }
 
@@ -1660,7 +1850,7 @@ namespace Orthanc
       std::unique_ptr<ImageAccessor> decoded;
       try
       {
-        decoded.reset(dicom.GetParsedDicomFile().DecodeFrame(frameIndex));
+        decoded.reset(dicom.DecodeFrame(frameIndex));
       }
       catch (OrthancException& e)
       {
@@ -1699,7 +1889,7 @@ namespace Orthanc
 
     if (builtinDecoderTranscoderOrder_ == BuiltinDecoderTranscoderOrder_After)
     {
-      return dicom.GetParsedDicomFile().DecodeFrame(frameIndex);
+      return dicom.DecodeFrame(frameIndex);
     }
     else
     {
@@ -1712,9 +1902,8 @@ namespace Orthanc
                                                  size_t size,
                                                  unsigned int frameIndex)
   {
-    DicomInstanceToStore instance;
-    instance.SetBuffer(dicom, size);
-    return DecodeDicomFrame(instance, frameIndex);
+    std::unique_ptr<DicomInstanceToStore> instance(DicomInstanceToStore::CreateFromBuffer(dicom, size));
+    return DecodeDicomFrame(*instance, frameIndex);
   }
   
 
