@@ -109,6 +109,12 @@ namespace Orthanc
     TransferSyntaxGroup_H265    // New in Orthanc 1.9.0
   };
 
+  enum TransactionType
+  {
+    TransactionType_ReadOnly,
+    TransactionType_ReadWrite
+  };
+
 
   /**
    * WARNING: Do not change the explicit values in the enumerations
@@ -119,7 +125,7 @@ namespace Orthanc
   enum GlobalProperty
   {
     GlobalProperty_DatabaseSchemaVersion = 1,   // Unused in the Orthanc core as of Orthanc 0.9.5
-    GlobalProperty_FlushSleep = 2,
+    GlobalProperty_FlushSleep = 2,              // Unused in the Orthanc core if Orthanc > 1.9.1
     GlobalProperty_AnonymizationSequence = 3,
     GlobalProperty_JobsRegistry = 5,
     GlobalProperty_GetTotalSizeIsFast = 6,      // New in Orthanc 1.5.2
@@ -149,12 +155,13 @@ namespace Orthanc
     MetadataType_ModifiedFrom = 5,
     MetadataType_AnonymizedFrom = 6,
     MetadataType_LastUpdate = 7,
-    MetadataType_Instance_Origin = 8,          // New in Orthanc 0.9.5
-    MetadataType_Instance_TransferSyntax = 9,  // New in Orthanc 1.2.0
-    MetadataType_Instance_SopClassUid = 10,    // New in Orthanc 1.2.0
-    MetadataType_Instance_RemoteIp = 11,       // New in Orthanc 1.4.0
-    MetadataType_Instance_CalledAet = 12,      // New in Orthanc 1.4.0
-    MetadataType_Instance_HttpUsername = 13,   // New in Orthanc 1.4.0
+    MetadataType_Instance_Origin = 8,            // New in Orthanc 0.9.5
+    MetadataType_Instance_TransferSyntax = 9,    // New in Orthanc 1.2.0
+    MetadataType_Instance_SopClassUid = 10,      // New in Orthanc 1.2.0
+    MetadataType_Instance_RemoteIp = 11,         // New in Orthanc 1.4.0
+    MetadataType_Instance_CalledAet = 12,        // New in Orthanc 1.4.0
+    MetadataType_Instance_HttpUsername = 13,     // New in Orthanc 1.4.0
+    MetadataType_Instance_PixelDataOffset = 14,  // New in Orthanc 1.9.0
 
     // Make sure that the value "65535" can be stored into this enumeration
     MetadataType_StartUser = 1024,
