@@ -195,7 +195,7 @@ namespace Orthanc
     virtual void Close() = 0;
 
     virtual void AddInstance(const std::string& instanceId,
-                             const FileInfo& dicom) = 0;
+                             uint64_t uncompressedSize) = 0;
   };
 
 
@@ -205,11 +205,12 @@ namespace Orthanc
     struct Instance
     {
       std::string  id_;
-      FileInfo     dicom_;
+      uint64_t     uncompressedSize_;
 
       Instance(const std::string& id,
-               const FileInfo& dicom) : 
-        id_(id), dicom_(dicom)
+               uint64_t uncompressedSize) : 
+        id_(id),
+        uncompressedSize_(uncompressedSize)
       {
       }
     };
@@ -228,9 +229,10 @@ namespace Orthanc
       if (level_ == ResourceType_Instance)
       {
         FileInfo tmp;
-        if (index.LookupAttachment(tmp, id, FileContentType_Dicom))
+        int64_t revision;  // ignored
+        if (index.LookupAttachment(tmp, revision, id, FileContentType_Dicom))
         {
-          instances_.push_back(Instance(id, tmp));
+          instances_.push_back(Instance(id, tmp.GetUncompressedSize()));
         }
       }
       else
@@ -335,7 +337,7 @@ namespace Orthanc
         for (std::list<Instance>::const_iterator 
                it = instances_.begin(); it != instances_.end(); ++it)
         {
-          visitor.AddInstance(it->id_, it->dicom_);
+          visitor.AddInstance(it->id_, it->uncompressedSize_);
         }          
       }
       else
@@ -370,7 +372,6 @@ namespace Orthanc
       Type          type_;
       std::string   filename_;
       std::string   instanceId_;
-      FileInfo      info_;
 
     public:
       explicit Command(Type type) :
@@ -389,12 +390,10 @@ namespace Orthanc
         
       Command(Type type,
               const std::string& filename,
-              const std::string& instanceId,
-              const FileInfo& info) :
+              const std::string& instanceId) :
         type_(type),
         filename_(filename),
-        instanceId_(instanceId),
-        info_(info)
+        instanceId_(instanceId)
       {
         assert(type_ == Type_WriteInstance);
       }
@@ -422,7 +421,7 @@ namespace Orthanc
 
             try
             {
-              context.ReadAttachment(content, info_);
+              context.ReadDicom(content, instanceId_);
             }
             catch (OrthancException& e)
             {
@@ -577,11 +576,11 @@ namespace Orthanc
 
     void AddWriteInstance(const std::string& filename,
                           const std::string& instanceId,
-                          const FileInfo& info)
+                          uint64_t uncompressedSize)
     {
-      commands_.push_back(new Command(Type_WriteInstance, filename, instanceId, info));
+      commands_.push_back(new Command(Type_WriteInstance, filename, instanceId));
       instancesCount_ ++;
-      uncompressedSize_ += info.GetUncompressedSize();
+      uncompressedSize_ += uncompressedSize;
     }
 
     bool IsZip64() const
@@ -695,13 +694,13 @@ namespace Orthanc
     }
 
     virtual void AddInstance(const std::string& instanceId,
-                             const FileInfo& dicom) ORTHANC_OVERRIDE
+                             uint64_t uncompressedSize) ORTHANC_OVERRIDE
     {
       char filename[24];
       snprintf(filename, sizeof(filename) - 1, instanceFormat_, counter_);
       counter_ ++;
 
-      commands_.AddWriteInstance(filename, instanceId, dicom);
+      commands_.AddWriteInstance(filename, instanceId, uncompressedSize);
     }
   };
 
@@ -710,14 +709,11 @@ namespace Orthanc
   {
   private:
     ZipCommands&    commands_;
-    ServerContext&  context_;
     unsigned int    counter_;
 
   public:
-    MediaIndexVisitor(ZipCommands& commands,
-                      ServerContext& context) :
+    explicit MediaIndexVisitor(ZipCommands& commands) :
       commands_(commands),
-      context_(context),
       counter_(0)
     {
     }
@@ -732,13 +728,13 @@ namespace Orthanc
     }
 
     virtual void AddInstance(const std::string& instanceId,
-                             const FileInfo& dicom) ORTHANC_OVERRIDE
+                             uint64_t uncompressedSize) ORTHANC_OVERRIDE
     {
       // "DICOM restricts the filenames on DICOM media to 8
       // characters (some systems wrongly use 8.3, but this does not
       // conform to the standard)."
       std::string filename = "IM" + boost::lexical_cast<std::string>(counter_);
-      commands_.AddWriteInstance(filename, instanceId, dicom);
+      commands_.AddWriteInstance(filename, instanceId, uncompressedSize);
 
       counter_ ++;
     }
@@ -748,7 +744,6 @@ namespace Orthanc
   class ArchiveJob::ZipWriterIterator : public boost::noncopyable
   {
   private:
-    TemporaryFile&                          target_;
     ServerContext&                          context_;
     ZipCommands                             commands_;
     std::unique_ptr<HierarchicalZipWriter>  zip_;
@@ -756,18 +751,17 @@ namespace Orthanc
     bool                                    isMedia_;
 
   public:
-    ZipWriterIterator(TemporaryFile& target,
+    ZipWriterIterator(const TemporaryFile& target,
                       ServerContext& context,
                       ArchiveIndex& archive,
                       bool isMedia,
                       bool enableExtendedSopClass) :
-      target_(target),
       context_(context),
       isMedia_(isMedia)
     {
       if (isMedia)
       {
-        MediaIndexVisitor visitor(commands_, context);
+        MediaIndexVisitor visitor(commands_);
         archive.Expand(context.GetIndex());
 
         commands_.AddOpenDirectory(MEDIA_IMAGES_FOLDER);        
